# -*- encoding: utf-8 -*-
#
# A scripting wrapper for SABnzbd's Post Processing Scripting
#
# Copyright (C) 2017 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
"""
This class was intended to make writing SABnzbd Post Process Scripts easier to
manage and write by handling the common error handling and simplify commont
tasks.

Some inline documentation was based on content provided at:
   - https://sabnzbd.org/wiki/scripts/post-processing-scripts


from nzbget import SABPostProcessScript

# Now define your class while inheriting the rest
class MySABPostProcessScript(SABPostProcessScript):
    def main(self, *args, **kwargs):

        # Version Checking, Environment Variables Present, etc
        if not self.validate():
            # No need to document a failure, validate will do that
            # on the reason it failed anyway
            return False

        # write all of your code here you would have otherwise put in the
        # script

        # All system environment variables (SAB_.*)

        print('SCRIPT (The current script is: %s' % self.get('SCRIPT'))
        print('NZO_ID %s' self.get('NZO_ID'))
        print('FILENAME %s' self.get('FILENAME'))
        print('PP_STATUS %s' self.get('PP_STATUS'))
        # ... etc

        # Set any variable you want by any key.  Note that if you use
        # keys that were defined by the system (such as CAT, DIRECTORY,
        # etc, you may have some undesirable results.  Try to avoid reusing
        # system variables already defined (identified above):
        self.set('MY_KEY', 'MY_VALUE')

        # You can fetch it back; this will also set an entry in  the
        # sqlite database for each hash references that can be pulled from
        # another script that simply calls self.get('MY_KEY')
        print(self.get('MY_KEY')) # prints MY_VALUE

        # You can also use push() which is similar to set()
        # except that it interacts with the NZBGet Server and does not use
        # the sqlite database. This can only be reached across other
        # scripts if the calling application is NZBGet itself
        self.push('ANOTHER_KEY', 'ANOTHER_VALUE')

        # You can still however locally retrieve what you set using push()
        # with the get() function
        print(self.get('ANOTHER_KEY')) # prints ANOTHER_VALUE

        # Returns have been made easy.  Just return:
        #   * True if everything was successful
        #   * False if there was a problem

        return True

# Call your script as follows:
if __name__ == "__main__":
    from sys import exit

    # Create an instance of your Script
    myscript = MySABPostProcessScript()

    # call run() and exit() using it's returned value
    exit(myscript.run())
"""

import re
from os import chdir
from os import unlink
from os import environ
from os.path import isdir
from os.path import isfile
from os.path import join
from os.path import splitext
from os.path import basename
from os.path import abspath

# NZB-File Compression Handling
import gzip
from tempfile import mkstemp

# Relative Includes
from .ScriptBase import ScriptBase
from .ScriptBase import SAB_ENVIRO_ID
from .ScriptBase import SCRIPT_MODE
from .ScriptBase import NZBGET_BOOL_FALSE
from .PostProcessCommon import OBFUSCATED_PATH_RE
from .PostProcessCommon import OBFUSCATED_FILE_RE

from .Utils import os_path_split as split


class TOTAL_STATUS(object):
    """Cumulative (Total) Status of NZB Processing
    """
    # everything OK
    SUCCESS = 'SUCCESS'
    # download is damaged but probably can be repaired; user intervention is
    # required;
    WARNING = 'WARNING'
    # download has failed or a serious error occurred during
    # post-processing (unpack, par);
    FAILURE = 'FAILURE'
    # download was deleted; post-processing scripts are usually not called in
    # this case; however it's possible to force calling scripts with command
    # "post-process again".
    DELETED = 'DELETED'


class PP_STATUS(object):
    """This provides the Post Process Status of SABNzbd set in the
    SAB_PP_STATUS environment variable
    """
    # checked and successfully repaired
    SUCCESS = 0
    # verification failed
    VERIFY_FAIL = 1
    # unpacking failed
    UNPACK_FAIL = 2
    # both verification and vunpack failed
    VERIFY_UNPACK_FAIL = 3
    # Failed Post Processing
    FAILURE = -1


class SABPostProcessScript(ScriptBase):
    """POST PROCESS mode is called after the unpack stage
    """
    def __init__(self, *args, **kwargs):
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Multi-Script Support
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        if not hasattr(self, 'script_dict'):
            # Only define once
            self.script_dict = {}
        self.script_dict[SCRIPT_MODE.SABNZBD_POSTPROCESSING] = self

        # Used to track a temporary NZB-File if we need one.
        self._sab_temp_nzb = None

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Initialize Parent
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        super(SABPostProcessScript, self).__init__(*args, **kwargs)

    def sabnzbd_postprocess_init(self, *args, **kwargs):
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Fetch Script Specific Arguments
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        directory = kwargs.get('directory')
        nzbname = kwargs.get('nzbname')
        nzbfilename = kwargs.get('nzbfilename')
        category = kwargs.get('category')
        status = kwargs.get('status')
        parse_nzbfile = kwargs.get('parse_nzbfile', True)
        use_database = kwargs.get('use_database', True)

        # self.directory
        # This is the path to the destination directory for downloaded files.
        if directory is None:
            self.directory = environ.get(
                '%sDIRECTORY' % SAB_ENVIRO_ID,
            )
            _final_directory = environ.get(
                '%sCOMPLETE_DIR' % SAB_ENVIRO_ID,
            )
            if self.directory and not isdir(self.directory):
                if _final_directory and isdir(_final_directory):
                    # adjust path
                    self.directory = _final_directory

            elif _final_directory and isdir(_final_directory):
                # adjust path
                self.directory = _final_directory
        else:
            self.directory = directory

        if self.directory:
            self.directory = abspath(self.directory)

        # self.nzbname
        # User-friendly name of processed nzb-file as it is displayed by the
        # program.  The file path and extension are removed.  If download was
        # renamed, this parameter reflects the new name.
        if nzbname is None:
            self.nzbname = environ.get(
                '%sFILENAME' % SAB_ENVIRO_ID,
            )
        else:
            self.nzbname = nzbname

        # self.nzbfilename
        # Name of processed nzb-file. If the file was added from incoming
        # nzb-directory, this is a full file name, including path and
        # extension. If the file was added from web-interface, it's only the
        # file name with extension. If the file was added via RPC-API (method
        # append), this can be any string but the use of actual file name is
        # recommended for developers.
        if nzbfilename is None:
            self.nzbfilename = environ.get(
                '%sORIG_NZB_GZ' % SAB_ENVIRO_ID,
            )
            if not self.nzbfilename:
                self.nzbfilename = environ.get(
                    '%sNZBNAME' % SAB_ENVIRO_ID,
                )
            # Fallback Check
            if not self.nzbfilename:
                self.nzbfilename = environ.get(
                    '%sURL' % SAB_ENVIRO_ID,
                )
            # last resort because we want to have this variable defined!
            if not self.nzbfilename:
                self.nzbfilename = self.nzbname

        else:
            self.nzbfilename = nzbfilename

        self.nzbfilename = self.handle_nzbfile(self.nzbfilename)

        # self.category
        # Category assigned to nzb-file (can be empty string).
        if category is None:
            self.category = environ.get(
                '%sCAT' % SAB_ENVIRO_ID,
            )
        else:
            self.category = category

        # self.status
        if status is None:
            self.status = environ.get(
                '%sPP_STATUS' % SAB_ENVIRO_ID,
            )
        else:
            self.status = status

        try:
            self.status = int(self.status)

        except (ValueError, TypeError):
            self.status = PP_STATUS.FAILURE

        _err = environ.get(
            '%sFAIL_MSG' % SAB_ENVIRO_ID,
        )

        self.logger.debug('SABnzbd PP Status: %s' % str(self.status))
        if _err:
            self.logger.debug('SABnzbd PP Error Message: %s' % str(_err))
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Error Handling
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        if self.nzbfilename:
            # absolute path names
            self.nzbfilename = abspath(self.nzbfilename)

            if parse_nzbfile:
                # Initialize information fetched from NZB-File
                # We intentionally allow existing nzbheaders to over-ride
                # any found in the nzbfile
                self.nzbheaders = dict(
                    self.parse_nzbfile(
                        self.nzbfilename, check_queued=True)\
                )

        if self.directory:
            # absolute path names
            self.directory = abspath(self.directory)

        if not (self.directory and isdir(self.directory)):
            self.logger.debug('Process directory is missing: %s' % \
                self.directory)
        else:
            try:
                chdir(self.directory)
            except OSError:
                self.logger.debug('Process directory is not accessible: %s' % \
                    self.directory)

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Enforce system/global variables for script processing
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        self.system['DIRECTORY'] = self.directory
        if self.directory is not None:
            environ['%sDIRECTORY' % SAB_ENVIRO_ID] = \
                self.directory

        self.system['FILENAME'] = self.nzbname
        if self.nzbname is not None:
            environ['%sFILENAME' % SAB_ENVIRO_ID] = \
                self.nzbname

        self.system['NZBNAME'] = self.nzbfilename
        if self.nzbfilename is not None:
            environ['%sNZBNAME' % SAB_ENVIRO_ID] = \
                self.nzbfilename

        self.system['CAT'] = self.category
        if self.category is not None:
            environ['%sCAT' % SAB_ENVIRO_ID] = \
                self.category

        self.system['PP_STATUS'] = str(self.status)
        if self.status is not None:
            environ['%sPP_STATUS' % SAB_ENVIRO_ID] = \
                str(self.status)

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Create Database for set() and get() operations
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        if use_database:
            # database_key is inherited in the parent class
            # future calls of set() and get() will allow access
            # to the database now
            try:
                self.database_key = \
                        self.get('NZO_ID', basename(self.nzbfilename))
            except AttributeError:
                pass

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Debug Flag Check
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def sabnzbd_postprocess_debug(self, *args, **kwargs):
        """Uses the environment variables to detect if debug mode is set
        """
        return self.parse_bool(
            environ.get('%sDEBUG' % SAB_ENVIRO_ID, NZBGET_BOOL_FALSE),
        )

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Sanity
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def sabnzbd_postprocess_sanity_check(self, *args, **kwargs):
        """Sanity checking to ensure this really is a SAB Post-Process Script
        """
        return ('%sCOMPLETE_DIR' % SAB_ENVIRO_ID in environ)

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Validatation
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def sabnzbd_postprocess_validate(self, keys=None, min_version=2,
                                     *args, **kwargs):
        """validate against environment variables
        """

        is_okay = super(SABPostProcessScript, self)._validate(
            keys=keys,
            min_version=min_version,
        )

        if min_version >= 2:
            required_opts = set((
                'COMPLETE_DIR',
                'PP_STATUS',
                'VERSION',
            ))
            found_opts = set(self.system) & required_opts
            if found_opts != required_opts:
                missing_opts = list(required_opts ^ found_opts)
                self.logger.error(
                    'Validation - (v2.x) Directives not set: %s' % \
                      ', '.join(missing_opts),
                )
                is_okay = False

        return is_okay

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Health Check
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def sabnzbd_postprocess_health_check(self, is_unpacked=True,
                                         has_archive=False, *args, **kwargs):
        """Similar to validate, except some scripts don't need to
        out right fail if the download health is bad. Some might just
        return silently, others may try to correct the health
        """

        is_okay = super(SABPostProcessScript, self)\
                ._health_check(*args, **kwargs)

        if self.status != PP_STATUS.SUCCESS:
            is_okay = False

        return is_okay

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # File Retrieval
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def sabnzbd_postprocess_get_files(self, search_dir=None, *args, **kwargs):
        """a wrapper to the get_files() function defined in the inherited class
           the only difference is the search_dir automatically uses the
           defined download `directory` as a default (if not specified).
        """
        if search_dir is None:
            search_dir = self.directory

        return super(SABPostProcessScript, self)._get_files(
            search_dir=search_dir, *args, **kwargs)

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Retrieve Statistics
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def get_statistics(self, nzbid=None):
        """
        Returns the download statistics (via the API)

        The result is returned in an easy to interpret dictionary like so
        {
            'download_time_sec': 40.0,
            'download_avg': 2.0,
            'bytes_downloaded': 2.0,
            'download_total': 2.0,
        }

        If an error occurs, then 'None' is returned.
        """

        return {
            # File Download Total Time (in seconds)
            'download_time_sec': self.get('DOWNLOAD_TIME'),
            # File Download Average Transfer Speed
            'download_avg': self.get('AVG_BPS'),
            # How many bytes were recieved (can be more than tried, due to
            # overhead)
            'bytes_downloaded': self.get('BYTES_DOWNLOADED'),
            'download_total': self.get('BYTES'),
        }

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Obfuscation Handling
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def deobfuscate(self, filename, ref_dir=None, ref_nzbfile=None):
        """attempts to detect and update
        """

        if ref_dir is None:
            ref_dir = self.directory

        if ref_nzbfile is None:
            ref_nzbfile = self.nzbfilename

        if filename[0:len(ref_dir):] == ref_dir:
            new_name = filename[len(ref_dir)+1:]
            self.logger.debug(
                'Deobfuscate - Stripped filename down to: %s' % new_name,
            )
        else:
            new_name = filename

        parts = split(new_name)
        part_removed = 0
        for x in range(0, len(parts)-1):
            fn = parts[x]
            if OBFUSCATED_PATH_RE.match(fn):
                self.logger.info(
                    'Detected obfuscated directory name %s,' % fn +
                    ' removing from path',
                )
                parts[x] = None
                part_removed += 1

        if OBFUSCATED_FILE_RE.match(basename(filename)):
            self.logger.info(
                'Detected obfuscated filename %s,' % basename(filename) +
                ' removing from path')
            parts[len(parts)-1] = '-' + splitext(filename)[1]
            part_removed += 1

        if part_removed < len(parts):
            new_name = ''
            for x in range(0, len(parts)):
                if parts[x] is not None:
                    new_name = join(new_name, parts[x])
            return new_name

        if OBFUSCATED_FILE_RE.match(new_name):
            new_name = ''

        # Check out NZB-Filename
        if len(self.nzb_items()):
            self.logger.info(
                'All file path parts are obfuscated, using obfuscated ' +
                'NZB-Headers',
            )

            # Fetch category
            category = self.nzb_get('category', '')\
                .split(' ')[0].lower()
            subcategory = self.nzb_get('category', '')\
                .split(' ')[-1].lower()

            if self.nzb_get('name'):
                # We can pick from the nzb headers
                nzb_name = self.nzb_get('name')
                new_name = join(ref_dir, '%s%s' % (
                    re.sub('[\s]+', '.', nzb_name),
                    splitext(basename(new_name))[1],
                ))

            elif category[0:5] == 'movie' and \
                    self.nzb_get('propername'):
                nzb_name = self.nzb_get('propername')

                if self.nzb_get('movieyear'):
                    nzb_name += '(%s)' % self.nzb_get('movieyear')

                new_name = join(ref_dir, '%s%s' % (
                    re.sub('[\s]+', '.', nzb_name),
                    splitext(basename(new_name))[1],
                ))

            elif category == 'tv' and \
                    self.nzb_get('propername'):
                nzb_name = self.nzb_get('propername')
                if self.nzb_get('episodename'):
                    nzb_name += '-%s' % \
                        self.nzb_get('episodename')

                if subcategory == 'hd':
                    nzb_name += '-HDTV'

                new_name = join(ref_dir, '%s%s' % (
                    re.sub('[\s]+', '.', nzb_name),
                    splitext(basename(new_name))[1],
                ))

            elif self.nzb_get('propername'):
                nzb_name = self.nzb_get('propername')
                new_name = join(ref_dir, '%s%s' % (
                    re.sub('[\s]+', '.', nzb_name),
                    splitext(basename(filename))[1],
                ))
            else:
                # No possible
                new_name = ''

        if new_name:
            self.logger.debug(
                'Deobfuscate - Generated filename: %s' % new_name,
            )
            return new_name

        # we're running out of new names :)... try the NZB-FileName
        if ref_nzbfile and not OBFUSCATED_FILE_RE.match(basename(ref_nzbfile)):
            self.logger.info(
                'All file path parts are obfuscated, using NZB-FileName',
            )
            new_name = join(ref_dir, '%s%s' % (
                re.sub('[\s]+', '.',
                       splitext(basename(ref_nzbfile))[0]),
                splitext(basename(filename))[1],
            ))

        else:
            self.logger.info('Deobfuscation is not possible')
            return filename

        self.logger.debug('Deobfuscate - Generated filename: %s' % new_name)
        return new_name

    def handle_nzbfile(self, nzbfile):
        """
        Takes an nzbfile and if it's compressed it creates a temporary
        uncompressed version we can reference. The function returns
        an uncompressed NZB-File if it can (obviously depends on what was
        passed in.
        """

        if not nzbfile:
            # Nothing we can do
            return ''

        # Handle .gz compressed NZB-Files and update our pointer accordingly
        if not isfile(nzbfile):
            # Nothing we can do; return what we started with
            return nzbfile

        # Extract our filename
        result = re.match('^(?P<filename>.+\.nzb)\.gz$', nzbfile, re.I)
        if not result:
            # We're done; that was easy
            return nzbfile

        # Extract our file into a temporary directory
        fo, self._sab_temp_nzb = mkstemp(dir=self.tempdir, suffix='.nzb')

        # Our file descriptors
        fi = None
        try:
            fi = gzip.open(nzbfile, "rb")

        except Exception:
            # Can't open the file
            return nzbfile

        try:
            fo = open(self._sab_temp_nzb, 'wb')
        except Exception:
            # Can't open the file
            return nzbfile

        try:
            while 1:
                buf = fi.read(16384)
                if not buf:
                    # We're Done
                    break
                fo.write(buf)

        except Exception:
            # oh well..
            unlink(self._sab_temp_nzb)
            self._sab_temp_nzb = None

        try:
            fo.close()
        except Exception:
            pass

        try:
            fi.close()
        except Exception:
            pass

        return self._sab_temp_nzb

    def sabnzbd_postprocess_close(self):
        """
        Allow the graceful handling of our temporary NZB-File
        """
        if self._sab_temp_nzb and isfile(self._sab_temp_nzb):
            try:
                # Cleanup
                unlink(self._sab_temp_nzb)
                self._sab_temp_nzb = None
            except:
                # we tried...
                pass



# -*- encoding: utf-8 -*-
#
# A Test Suite (for nose) for the ScriptBase Class
#
# Copyright (C) 2014-2017 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
import os
import sys
import re
from os.path import join
from os.path import isfile
from urllib import unquote

from TestBase import TestBase
from TestBase import TEMP_DIRECTORY

from nzbget.ScriptBase import ScriptBase
from nzbget.ScriptBase import SYS_ENVIRO_ID
from nzbget.ScriptBase import EXIT_CODE
from nzbget.ScriptBase import SHELL_EXIT_CODE
from nzbget.ScriptBase import CFG_ENVIRO_ID
from nzbget.ScriptBase import Health
from nzbget.Logger import VERY_VERBOSE_DEBUG

from shutil import rmtree
from os import makedirs
from threading import Thread, Lock
from time import sleep

import StringIO

SEARCH_DIR = join(TEMP_DIRECTORY, 'file_listing')


class TestScriptBase(TestBase):
    def setUp(self):
        # common
        super(TestScriptBase, self).setUp()

        # Create some environment variables
        os.environ['%sTEMPDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sSCRIPTDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY

        # ensure directory doesn't exist
        try:
            rmtree(SEARCH_DIR)
        except:
            pass
        makedirs(SEARCH_DIR)

    def tearDown(self):
        if '%sTEMPDIR' % SYS_ENVIRO_ID in os.environ:
            del os.environ['%sTEMPDIR' % SYS_ENVIRO_ID]
        if '%sSCRIPTDIR' % SYS_ENVIRO_ID in os.environ:
            del os.environ['%sSCRIPTDIR' % SYS_ENVIRO_ID]
        if '%sDEBUG' % CFG_ENVIRO_ID in os.environ:
            del os.environ['%sDEBUG' % CFG_ENVIRO_ID]

        try:
            rmtree(SEARCH_DIR)
        except:
            pass

        # common
        super(TestScriptBase, self).tearDown()

    def test_main_returns(self):

        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.run() == SHELL_EXIT_CODE.SUCCESS

        class ScriptExceptionExit(ScriptBase):
            def main(self, *args, **kwargs):
                raise TypeError

        script = ScriptExceptionExit()
        assert script.run() == SHELL_EXIT_CODE.FAILURE

    def test_nzbset_and_nzbget(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)
        KEY = 'MY_NZBVAR'
        KEYL = 'my_nzbvar'
        VALUE = 'MY_NZBVALUE'
        ADJUSTED_VALUE = 'MY_NEW_NZBVALUE'

        # Value does not exist yet
        assert script.nzb_get(KEY) is None
        assert script.nzb_get(KEY, ADJUSTED_VALUE) == ADJUSTED_VALUE
        assert script.nzb_get(KEYL, ADJUSTED_VALUE) == ADJUSTED_VALUE
        assert script.nzb_set(KEY, VALUE) is True
        assert script.nzb_get(KEY, ADJUSTED_VALUE) == VALUE
        assert script.nzb_get(KEYL, ADJUSTED_VALUE) == VALUE
        assert script.nzb_unset(KEYL) is True
        assert script.nzb_get(KEY, ADJUSTED_VALUE) == ADJUSTED_VALUE

        # Reset with content
        assert script.nzb_set(KEY, ADJUSTED_VALUE) is True
        assert script.nzb_get(KEY, VALUE) == ADJUSTED_VALUE

        # Same as nzb_unset
        assert script.nzb_set(KEY, None) is True
        assert script.nzb_get(KEY, VALUE) == VALUE

    def test_set_and_get(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)

        KEY = 'MY_VAR'
        VALUE = 'MY_VALUE'
        ADJUSTED_VALUE = 'MY_NEW_VALUE'

        # Value does not exist yet
        assert script.get(KEY) is None
        assert script.get(KEY, ADJUSTED_VALUE) == ADJUSTED_VALUE
        assert script.set(KEY, VALUE) is True
        assert script.get(KEY, ADJUSTED_VALUE) == VALUE

        # Remove a variable
        assert script.unset(KEY) is True
        # we'll get the default value now since the key wasn't
        # found.
        assert script.get(KEY, ADJUSTED_VALUE) == ADJUSTED_VALUE

        # Reset our value but be careful not to set the environment
        # or set the database
        assert script.set(KEY, VALUE, use_env=False, use_db=False) is True

        # observe content is not carried over
        script = ScriptBase(
            logger=False, debug=VERY_VERBOSE_DEBUG, database_key='test')
        assert script.get(KEY) is None

        # The below line will include environment variables when
        # preforming the set and not the database
        assert script.set(KEY, VALUE, use_db=False) is True
        # If we destroy our instance
        del script

        # and reinitialize it; content should still carry from
        # the environment variable
        script = ScriptBase(
            logger=False, debug=VERY_VERBOSE_DEBUG, database_key='test')
        assert script.get(KEY) == VALUE
        assert script.unset(KEY) is True

        # Now we set the database by just the database and not
        # the environment variable
        assert script.set(KEY, ADJUSTED_VALUE, use_env=False) is True

        # Destroy the instance
        del script
        script = ScriptBase(
            logger=False, debug=VERY_VERBOSE_DEBUG, database_key='test')
        # We can still retrieve our adjusted value
        assert script.get(KEY) == ADJUSTED_VALUE
        # But not if we don't reference the database
        assert script.get(KEY, use_db=False) is None

        # if we don't set a database key, then there is no way for a
        # database connection to be established since we have no
        # container to work out of. This can be emulated by just creating
        # a ScriptBase object without a database_key set and we won't
        # connect to the database
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)

        # Now we set the database by just the database and not
        # the environment variable. Even though use_db defaults to
        # true, we won't write anythign to it since we can't establish
        # a connection to the database.

        # If we set a key...
        assert script.set(KEY, VALUE, use_env=False) is True

        # We can check that it was set
        assert script.get(KEY) == VALUE
        del script

        # But content won't carry over... infact, we'll fetch the last
        # key we fetched if we use the same database_key (allowing us
        # to access the same container as before)
        script = ScriptBase(
            logger=False, debug=VERY_VERBOSE_DEBUG, database_key='test')
        # We can still retrieve our adjusted value
        assert script.get(KEY) == ADJUSTED_VALUE

        del script
        # But we won't get anything under a different container
        script = ScriptBase(
            logger=False, debug=VERY_VERBOSE_DEBUG, database_key='ugh!')
        assert script.get(KEY) is None

    def test_file_listings_with_file(self):

        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.get_files(search_dir=[SEARCH_DIR, ]) == {}

        # Create some temporary files to work with
        open(join(SEARCH_DIR, 'file.mkv'), 'w').close()
        open(join(SEARCH_DIR, 'file.par2'), 'w').close()
        open(join(SEARCH_DIR, 'file.PAR2'), 'w').close()
        open(join(SEARCH_DIR, 'file.txt'), 'w').close()
        open(join(SEARCH_DIR, 'sample.mp4'), 'w').close()
        open(join(SEARCH_DIR, 'sound.mp3'), 'w').close()
        open(join(SEARCH_DIR, 'NOEXTENSION'), 'w').close()

        files = script.get_files(
            search_dir=SEARCH_DIR,
        )
        assert len(files) == 7

        # Search File
        files = script.get_files(
            search_dir=join(SEARCH_DIR, 'file.mkv'),
        )
        assert len(files) == 1

        files = script.get_files(
            search_dir=join(SEARCH_DIR, 'file.mkv'),
            suffix_filter='.mov',
        )
        assert len(files) == 0

        files = script.get_files(
            search_dir=join(SEARCH_DIR, 'file.mkv'),
            suffix_filter='.mkv',
        )
        assert len(files) == 1

        files = script.get_files(
            search_dir=join(SEARCH_DIR, 'file.mkv'),
            suffix_filter='.mkv',
            prefix_filter='fi',
        )
        assert len(files) == 1

        files = script.get_files(
            search_dir=join(SEARCH_DIR, 'file.mkv'),
            suffix_filter='.mkv',
            prefix_filter='file',
        )
        assert len(files) == 1

        # Absolute match
        files = script.get_files(
            search_dir=join(SEARCH_DIR, 'file.mkv'),
            suffix_filter='file.mkv',
            prefix_filter='file.mkv',
        )
        assert len(files) == 1

        files = script.get_files(
            search_dir=join(SEARCH_DIR, 'file.mkv'),
            regex_filter='^file\.mkv$',
        )
        assert len(files) == 1

        files = script.get_files(
            search_dir=join(SEARCH_DIR, 'file.mkv'),
            suffix_filter='file.mkv',
            prefix_filter='file.mkv',
            regex_filter='^file\.mkv$',
        )
        assert len(files) == 1

    def test_file_listings_with_depth(self):

        subdirs = (
            'depth2/depth3/depth4/depth5',
            'depth2_a/depth3_a/depth4_a/depth5_a',
            'depth2_b/depth3_b/depth4_b/depth5_b',
            'depth2_c/depth3_c/depth4_c/depth5_c/depth6_c',
        )
        for _dir in subdirs:
            makedirs(join(SEARCH_DIR, _dir))

        # Create some temporary files to work with
        open(join(SEARCH_DIR, 'file.001'), 'w').close()
        open(join(SEARCH_DIR, 'depth2_c', 'depth3_c',
                  'depth4_c', 'depth5_c', 'depth6_c', 'file.006'), 'w').close()
        open(join(SEARCH_DIR, 'depth2', 'depth3', 'depth4',
                  'depth5', 'file.005'), 'w').close()
        open(join(SEARCH_DIR, 'depth2_a', 'depth3_a', 'depth4_a',
                  'depth5_a', 'file.005'), 'w').close()
        open(join(SEARCH_DIR, 'depth2_b', 'depth3_b', 'depth4_b',
                  'depth5_b', 'file.005'), 'w').close()
        open(join(SEARCH_DIR, 'depth2', 'depth3', 'depth4',
                  'file.004'), 'w').close()
        open(join(SEARCH_DIR, 'depth2', 'depth3',
                  'file.003'), 'w').close()
        open(join(SEARCH_DIR, 'depth2', 'file.002'), 'w').close()

        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)
        results = script.get_files(search_dir=SEARCH_DIR, max_depth=2)
        assert len(results) == 2
        assert join(SEARCH_DIR, 'depth2', 'file.002') in results.keys()
        assert join(SEARCH_DIR, 'file.001') in results.keys()

        # Recursion works a little differnetly, check that results
        # are the same:
        results = script.get_files(search_dir=[SEARCH_DIR, ], max_depth=2)
        assert len(results) == 2
        assert join(SEARCH_DIR, 'depth2', 'file.002') in results.keys()
        assert join(SEARCH_DIR, 'file.001') in results.keys()

        # This should fetch them all
        files = script.get_files(SEARCH_DIR)
        assert len(files) == 8

        # A list of directories should scan the same content, minus stuff at
        # the root
        files = script.get_files(
            [join(SEARCH_DIR, re.split('/', _dir)[0]) for _dir in subdirs])
        assert len(files) == 7

    def test_file_listings_ignored_directories(self):
        from nzbget.ScriptBase import SKIP_DIRECTORIES

        # Create some bad directories
        for _dir in SKIP_DIRECTORIES:
            makedirs(join(SEARCH_DIR, _dir))
            open(join(SEARCH_DIR, _dir, 'ignore_me.mkv'), 'w').close()

        # Create some temporary files to work with
        makedirs(join(SEARCH_DIR, 'good_dir'))
        open(join(SEARCH_DIR, 'good_file1.mkv'), 'w').close()
        open(join(SEARCH_DIR, 'good_file2.mkv'), 'w').close()
        open(join(SEARCH_DIR, 'good_dir', 'good_file3.mkv'), 'w').close()

        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)
        results = script.get_files(search_dir=SEARCH_DIR)
        assert len(results) == 3
        assert join(SEARCH_DIR, 'good_file1.mkv') in results.keys()
        assert join(SEARCH_DIR, 'good_file2.mkv') in results.keys()
        assert join(SEARCH_DIR, 'good_dir', 'good_file3.mkv') in \
            results.keys()

    def test_parse_url01(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)

        result = script.parse_url('http://hostname')
        assert result['schema'] == 'http'
        assert result['host'] == 'hostname'
        assert result['port'] is None
        assert result['user'] is None
        assert result['password'] is None
        assert result['fullpath'] is None
        assert result['path'] is None
        assert result['query'] is None
        assert result['url'] == 'http://hostname'
        assert result['qsd'] == {}

        result = script.parse_url('http://hostname/')
        assert result['schema'] == 'http'
        assert result['host'] == 'hostname'
        assert result['port'] is None
        assert result['user'] is None
        assert result['password'] is None
        assert result['fullpath'] == '/'
        assert result['path'] == '/'
        assert result['query'] is None
        assert result['url'] == 'http://hostname/'
        assert result['qsd'] == {}

        result = script.parse_url('hostname')
        assert result['schema'] == 'http'
        assert result['host'] == 'hostname'
        assert result['port'] is None
        assert result['user'] is None
        assert result['password'] is None
        assert result['fullpath'] is None
        assert result['path'] is None
        assert result['query'] is None
        assert result['url'] == 'http://hostname'
        assert result['qsd'] == {}

        result = script.parse_url('http://hostname////')
        assert result['schema'] == 'http'
        assert result['host'] == 'hostname'
        assert result['port'] is None
        assert result['user'] is None
        assert result['password'] is None
        assert result['fullpath'] == '/'
        assert result['path'] == '/'
        assert result['query'] is None
        assert result['url'] == 'http://hostname/'
        assert result['qsd'] == {}

        result = script.parse_url('http://hostname:40////')
        assert result['schema'] == 'http'
        assert result['host'] == 'hostname'
        assert result['port'] == 40
        assert result['user'] is None
        assert result['password'] is None
        assert result['fullpath'] == '/'
        assert result['path'] == '/'
        assert result['query'] is None
        assert result['url'] == 'http://hostname:40/'
        assert result['qsd'] == {}

        result = script.parse_url('HTTP://HoStNaMe:40/test.php')
        assert result['schema'] == 'http'
        assert result['host'] == 'HoStNaMe'
        assert result['port'] == 40
        assert result['user'] is None
        assert result['password'] is None
        assert result['fullpath'] == '/test.php'
        assert result['path'] == '/'
        assert result['query'] == 'test.php'
        assert result['url'] == 'http://HoStNaMe:40/test.php'
        assert result['qsd'] == {}

        result = script.parse_url('HTTPS://user@hostname/test.py')
        assert result['schema'] == 'https'
        assert result['host'] == 'hostname'
        assert result['port'] is None
        assert result['user'] == 'user'
        assert result['password'] is None
        assert result['fullpath'] == '/test.py'
        assert result['path'] == '/'
        assert result['query'] == 'test.py'
        assert result['url'] == 'https://user@hostname/test.py'
        assert result['qsd'] == {}

        result = script.parse_url('  HTTPS://///user@@@hostname///test.py  ')
        assert result['schema'] == 'https'
        assert result['host'] == 'hostname'
        assert result['port'] is None
        assert result['user'] == 'user'
        assert result['password'] is None
        assert result['fullpath'] == '/test.py'
        assert result['path'] == '/'
        assert result['query'] == 'test.py'
        assert result['url'] == 'https://user@hostname/test.py'
        assert result['qsd'] == {}

        result = script.parse_url(
            'HTTPS://user:password@otherHost/full///path/name/',
        )
        assert result['schema'] == 'https'
        assert result['host'] == 'otherHost'
        assert result['port'] is None
        assert result['user'] == 'user'
        assert result['password'] == 'password'
        assert result['fullpath'] == '/full/path/name/'
        assert result['path'] == '/full/path/name/'
        assert result['query'] is None
        assert result['url'] == \
            'https://user:password@otherHost/full/path/name/'
        assert result['qsd'] == {}

        # Handle garbage
        assert script.parse_url(None) is None

        result = script.parse_url(
            'mailto://user:password@otherHost/lead2gold@gmail.com' +
            '?from=test@test.com&name=Chris%20Caron&format=text'
        )
        assert result['schema'] == 'mailto'
        assert result['host'] == 'otherHost'
        assert result['port'] is None
        assert result['user'] == 'user'
        assert result['password'] == 'password'
        assert unquote(result['fullpath']) == '/lead2gold@gmail.com'
        assert result['path'] == '/'
        assert unquote(result['query']) == 'lead2gold@gmail.com'
        assert unquote(result['url']) == \
            'mailto://user:password@otherHost/lead2gold@gmail.com'
        assert len(result['qsd']) == 3
        assert 'name' in result['qsd']
        assert unquote(result['qsd']['name']) == 'Chris Caron'
        assert 'from' in result['qsd']
        assert unquote(result['qsd']['from']) == 'test@test.com'
        assert 'format' in result['qsd']
        assert unquote(result['qsd']['format']) == 'text'

        # Test Passwords with question marks ?; not supported
        result = script.parse_url(
            'http://user:pass.with.?question@host'
        )
        assert result is None
        result = script.parse_url(
            'http://user@host?pass=pass.with.?question'
        )

        assert result['schema'] == 'http'
        assert result['host'] == 'host'
        assert result['port'] is None
        assert result['user'] == 'user'
        assert result['password'] == 'pass.with.?question'
        assert result['fullpath'] is None
        assert result['path'] is None
        assert result['query'] is None
        assert unquote(result['url']) == 'http://user:pass.with.?question@host'
        assert len(result['qsd']) == 1
        assert unquote(result['qsd']['pass']) == 'pass.with.?question'

    def test_parse_list(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)

        # A simple single array entry (As str)
        results = script.parse_list('.mkv,.avi,.divx,.xvid,' +
                                    '.mov,.wmv,.mp4,.mpg,.mpeg,.vob,.iso')
        assert results == [
            '.divx', '.iso', '.mkv', '.mov', '.mpg',
            '.avi', '.mpeg', '.vob', '.xvid', '.wmv', '.mp4',
        ]

        # Now 2 lists with lots of duplicates and other delimiters
        results = script.parse_list(
            '.mkv,.avi,.divx,.xvid,.mov,.wmv,.mp4,.mpg .mpeg,.vob,,; ;',
            '.mkv,.avi,.divx,.xvid,' +
            '.mov        .wmv,.mp4;.mpg,.mpeg,.vob,.iso')
        assert results == [
            '.divx', '.iso', '.mkv', '.mov', '.mpg',
            '.avi', '.mpeg', '.vob', '.xvid', '.wmv', '.mp4',
        ]

        # Now a list with extras we want to add as strings
        # empty entries are removed
        results = script.parse_list([
            '.divx', '.iso', '.mkv', '.mov', '', '  ',
            '.avi', '.mpeg', '.vob', '.xvid', '.mp4',
        ],
            '.mov,.wmv,.mp4,.mpg',
        )
        assert results == [
            '.divx', '.wmv', '.iso', '.mkv', '.mov',
            '.mpg', '.avi', '.vob', '.xvid', '.mpeg', '.mp4',
        ]

    def test_parse_path_list01(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)

        # A simple single array entry (As str)
        results = script.parse_path_list(
            'C:\\test path with space\\and more spaces\\',
            'D:\\weird path\\more spaces\\ ',
            'D:\\weird path\\more spaces\\ G:\\save E:\\\\save_dir',
            'D:\\weird path\\more spaces\\ ',  # Duplicate is removed
            '  D:\\\\weird path\\more spaces\\\\ ',  # Duplicate is removed
            'relative path\\\\\\with\\crap\\\\ second_path\\more spaces\\ ',
            # Windows Network Paths
            '\\\\a\\network\\path\\\\ \\\\\\another\\nw\\\\path\\\\         ',
            # lists supported too
            ['relative path\\in\\list', 'second_path\\more spaces\\\\'],
            # Unsupported and ignored types
            None, 1, 4.3, True, False, -4000,
        )

        assert len(results) == 9
        assert 'D:\\weird path\\more spaces' in results
        assert 'E:\\save_dir' in results
        assert 'G:\\save' in results
        assert 'second_path\\more spaces' in results
        assert '\\\\a\\network\\path' in results
        assert '\\\\another\\nw\\path' in results
        assert 'relative path\\with\\crap' in results
        assert 'relative path\\in\\list' in results
        assert 'C:\\test path with space\\and more spaces' in results

    def test_parse_path_list02(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)

        # A simple single array entry (As str)
        results = script.parse_path_list(
            # 3 paths identified below
            '//absolute/path /another///absolute//path/// /',
            # A whole slew of duplicates and list inside list
            '/', ['/', '/', '//', '//////', ['/', ''], ],
            'relative/path/here',
            'relative/path/here/',
            'another/relative////path///',
        )
        assert len(results) == 5
        assert '/absolute/path' in results
        assert '/another/absolute/path' in results
        assert 'another/relative/path' in results
        assert '/' in results
        assert 'relative/path/here' in results

        results = script.parse_path_list(
            '/home/username/News/TVShows, /home/username/News/Movies',
        )
        assert len(results) == 2
        assert '/home/username/News/TVShows' in results
        assert '/home/username/News/Movies' in results

    def test_parse_path_list03(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)

        # A simple single array entry (As str)
        results = script.parse_path_list(
            '\\\\ht-pc\\htpc_5tb\\Media\\TV, ' +
            '\\\\ht-pc\\htpc_5tb\\Media\\Movies, ' +
            '\\\\ht-pc\\htpc_2tb\\Media\\TV, ' +
            '\\\\ht-pc\\htpc_2tb\\Media\\Movies',
        )
        assert len(results) == 4
        assert '\\\\ht-pc\\htpc_5tb\\Media\\TV' in results
        assert '\\\\ht-pc\\htpc_5tb\\Media\\Movies' in results
        assert '\\\\ht-pc\\htpc_2tb\\Media\\TV' in results
        assert '\\\\ht-pc\\htpc_2tb\\Media\\Movies' in results

    def test_parse_path_list04(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)

        # comma is a delimiter
        results = script.parse_path_list('path1, path2 ,path3,path4')
        assert len(results) == 4
        assert 'path1' in results
        assert 'path2' in results
        assert 'path3' in results
        assert 'path4' in results

    def test_parse_bool(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.parse_bool('Enabled', None) is True
        assert script.parse_bool('Disabled', None) is False
        assert script.parse_bool('Allow', None) is True
        assert script.parse_bool('Deny', None) is False
        assert script.parse_bool('Yes', None) is True
        assert script.parse_bool('YES', None) is True
        assert script.parse_bool('Always', None) is True
        assert script.parse_bool('No', None) is False
        assert script.parse_bool('NO', None) is False
        assert script.parse_bool('NEVER', None) is False
        assert script.parse_bool('TrUE', None) is True
        assert script.parse_bool('tRUe', None) is True
        assert script.parse_bool('FAlse', None) is False
        assert script.parse_bool('F', None) is False
        assert script.parse_bool('T', None) is True
        assert script.parse_bool('0', None) is False
        assert script.parse_bool('1', None) is True
        assert script.parse_bool('True', None) is True
        assert script.parse_bool('Yes', None) is True
        assert script.parse_bool(1, None) is True
        assert script.parse_bool(0, None) is False
        assert script.parse_bool(True, None) is True
        assert script.parse_bool(False, None) is False

        # only the int of 0 will return False since the function
        # casts this to a boolean
        assert script.parse_bool(2, None) is True
        # An empty list is still false
        assert script.parse_bool([], None) is False
        # But a list that contains something is True
        assert script.parse_bool(['value', ], None) is True

        # Use Default (which is False)
        assert script.parse_bool('OhYeah') is False
        # Adjust Default and get a different result
        assert script.parse_bool('OhYeah', True) is True

    def test_guesses(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)

        guess_dict = {
            'type': 'movie',
            'title': 'A Great Title',
            'year': 1998,
            'screenSize': '720p',
            'format': 'HD-DVD',
            'audioCodec': 'DTS',
            'videoCodec': 'h264',
            'releaseGroup': 'GREATTEAM',
            'BadEntry': 'Ignored',
        }

        # Keep a handle on the real standard output
        stdout = sys.stdout
        sys.stdout = StringIO.StringIO()
        script.push_guess(guess_dict)

        # extract data
        sys.stdout.seek(0)
        output = sys.stdout.read().strip()

        # return stdout back to how it was
        sys.stdout = stdout

        guess_keys = guess_dict.keys()
        guess_keys.remove('BadEntry')

        output = re.split('[\r\n]+', output)
        # Clean "" entry
        output = filter(bool, output)

        # Pushes are disabled in Standalone mode
        assert len(output) == 0

    def test_pushes(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)

        KEY = 'MY_VAR'
        VALUE = 'MY_VALUE'

        # Value doe snot exist yet
        assert script.get(KEY) is None

        # Keep a handle on the real standard output
        stdout = sys.stdout
        sys.stdout = StringIO.StringIO()
        script.push(KEY, VALUE)

        # extract data
        sys.stdout.seek(0)
        output = sys.stdout.read().strip()

        sys.stdout = stdout

        # Pushes are disabled in Standalone mode
        output = re.split('[\r\n]+', output)
        # Clean "" entry
        output = filter(bool, output)

        assert len(output) == 0
        assert script.get(KEY) == VALUE

    def test_validate(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.validate('TEMPDIR') is True
        # allow lowercase and mixed characters too
        assert script.validate('TempDir') is True

    def test_items(self):
        """see if we can retreive all our set variables using the items()
        function.
        """

        keypair = {
            'KEY1': 'val1',
            'KEY2': 'val2',
            'KEY3': 'val3',
            'KEY4': 'val4',
            'KEY5': 'val5',
        }

        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)
        for k, v in keypair.items():
            script.set(k, v)

        # Value doe snot exist yet
        for k, v in keypair.items():
            assert script.get(k) == v

        all_items = dict(script.items())
        for k, v in keypair.items():
            assert k in all_items
            assert script.get(k) == v

    def test_health(self):
        """ Test Health Checks
        """
        h = Health('%s/ALL' % Health.SUCCESS)
        assert h == (Health.SUCCESS, 'ALL')
        assert h.has_archive is False
        assert h.is_unpacked is True
        h = Health(('SUccESS', 'all'))
        assert h == (Health.SUCCESS, 'ALL')
        assert h.has_archive is False
        assert h.is_unpacked is True
        h = Health(Health.SUCCESS)
        assert h == (Health.SUCCESS, Health.DEFAULT_SUB)
        assert h.has_archive is False
        assert h.is_unpacked is True
        h = Health('BADKEYWORD')
        assert h == (Health.UNDEFINED, Health.DEFAULT_SUB)
        assert h.has_archive is True
        assert h.is_unpacked is True
        h = Health(Health.WARNING)
        assert h == (Health.WARNING, Health.DEFAULT_SUB)
        assert h.has_archive is True
        assert h.is_unpacked is False
        h = Health(Health.DELETED)
        assert h == (Health.DELETED, Health.DEFAULT_SUB)
        assert h.has_archive is False
        assert h.is_unpacked is False
        h = Health('')
        assert h == (Health.UNDEFINED, Health.DEFAULT_SUB)
        h = Health(None)
        assert h == (Health.UNDEFINED, Health.DEFAULT_SUB)

    def test_pid_file_control_01(self):
        """
        A Simple test that shows how multiple calls
        to is_unique_instance will not tamper with
        the PID-File or the process
        """

        # Define a simple script
        class SleepScript(ScriptBase):
            """
            A Simple sleeping script we'll thread for testing
            pid file generation
            """
            def main(self, *args, **kwargs):

                # Acquire lock
                self.lock.acquire()

                if not self.is_unique_instance(die_on_fail=False):
                    self.rlock.release()
                    self.lock.acquire()
                    return False

                if not self.is_unique_instance(die_on_fail=False):
                    self.rlock.release()
                    self.lock.acquire()
                    return False

                self.rlock.release()
                self.lock.acquire()
                return None

        def threaded_script(script):
            script._return_code = script.run()
            return

        # Script reference
        script = SleepScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        script.lock = Lock()
        script.rlock = Lock()
        script._return_code = None

        # Start locked
        script.lock.acquire()
        script.rlock.acquire()

        # Create our thread
        thread = Thread(target=threaded_script, args=(script, ))
        thread.start()

        # Let our script have control and await our turn
        script.lock.release()
        script.rlock.acquire()

        # Wait for our script to come close to exiting
        try:
            assert isfile(script.pidfile)

        except:
            # handle locks if we have a failure
            script.rlock.release()
            script.lock.release()
            thread.join()
            raise

        # Now let the threads exit gracefully
        script.lock.release()
        thread.join()

        # PID file tidys up nicely
        assert not isfile(script.pidfile)
        assert script._return_code == SHELL_EXIT_CODE.NONE

    def test_pid_file_control_02(self):
        """
        If the PID-File is pulled from under the process then
        it is treated as no longer being a unique process.

        We do things a little complicated when it isn't really nessisary.
        Hence this test uses threads to show it's possible. The main thing
        we're testing here however is that when we're done, we tidy up our
        own PID-File automatically.
        """

        # Define a simple script
        class SleepScript(ScriptBase):
            """
            A Simple sleeping script we'll thread for testing
            pid file generation
            """
            def main(self, *args, **kwargs):

                # Acquire lock
                self.lock.acquire()

                if not self.is_unique_instance(die_on_fail=False):
                    self.lock.acquire()
                    self.rlock.release()
                    return False

                # We want to pause here intentionally because
                # the test script is going to mess with the pid
                # file information for the lower processing
                self.rlock.release()
                self.lock.acquire()

                if not self.is_unique_instance(die_on_fail=False):
                    return False

                return None

        def threaded_script(script):
            script._return_code = script.run()
            return

        # Script reference
        script = SleepScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        script.lock = Lock()
        script.rlock = Lock()
        script._return_code = None

        # Start locked
        script.lock.acquire()
        script.rlock.acquire()

        # Create our thread
        thread = Thread(target=threaded_script, args=(script, ))
        thread.start()

        # Let our script have control and await our turn
        script.lock.release()

        script.rlock.acquire()
        # Wait for our script to come close to exiting
        try:
            assert isfile(script.pidfile)

        except:
            # handle locks if we have a failure
            script.lock.release()
            thread.join()
            raise

        # Remove this pidfile; this yanks the carpet from under it
        # causing it to treat the process as no longer being unique
        os.unlink(script.pidfile)

        script.lock.release()
        # Now let the threads exit gracefully
        thread.join()

        # PID file tidys up nicely
        assert not isfile(script.pidfile)
        assert script._return_code == SHELL_EXIT_CODE.FAILURE

    def test_pid_file_control_03(self):
        """
        This is the same as test_pid_file_control_02, we're basically
        going to change the modification time of the pid file which
        causes the script to no longer treat the process as unique

        We do things a little complicated when it isn't really nessisary.
        Hence this test uses threads to show it's possible. The main thing
        we're testing here however is that when there is a failure, the
        base script keeps the PID-File around if it's been tampered with.
        """

        # Define a simple script
        class SleepScript(ScriptBase):
            """
            A Simple sleeping script we'll thread for testing
            pid file generation
            """
            def main(self, *args, **kwargs):

                # Acquire lock
                self.lock.acquire()

                if not self.is_unique_instance(die_on_fail=False):
                    self.lock.acquire()
                    self.rlock.release()
                    return False

                # We want to pause here intentionally because
                # the test script is going to mess with the pid
                # file information for the lower processing
                self.rlock.release()
                self.lock.acquire()

                if not self.is_unique_instance(die_on_fail=False):
                    return False

                return None

        def threaded_script(script):
            script._return_code = script.run()
            return

        # Script reference
        script = SleepScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        script.lock = Lock()
        script.rlock = Lock()
        script._return_code = None

        # Start locked
        script.lock.acquire()
        script.rlock.acquire()

        # Create our thread
        thread = Thread(target=threaded_script, args=(script, ))
        thread.start()

        # Let our script have control and await our turn
        script.lock.release()

        script.rlock.acquire()
        # Wait for our script to come close to exiting
        try:
            assert isfile(script.pidfile)

        except:
            # handle locks if we have a failure
            script.lock.release()
            thread.join()
            raise

        # At least 1 second has to elapse so the changed modification
        # time will show through
        sleep(1)

        # touch our pid file (causes the modification time to shift)
        # thus becoming different then what we are currently set to
        with open(script.pidfile, 'a'):
            os.utime(script.pidfile, None)

        script.lock.release()
        # Now let the threads exit gracefully
        thread.join()

        # PID file tidys up nicely
        assert not isfile(script.pidfile)
        assert script._return_code == SHELL_EXIT_CODE.FAILURE

    def test_pid_file_control_04(self):
        """
        If our PID is different then what was written in the
        PID-File, this also causes us to abort
        """

        # Script reference
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.pidfile is None
        assert script.is_unique_instance(die_on_fail=False) is True
        assert script.is_unique_instance(die_on_fail=False) is True
        assert script.is_unique_instance(die_on_fail=False) is True
        assert isfile(script.pidfile)

        # Change our PID so it won't match what is in the
        # PID-File; this should trigger a failure
        script.pid = 0

        assert script.is_unique_instance(die_on_fail=False) is False

        # PID-File should still be present!!
        assert isfile(script.pidfile)

    def test_parse_regex(self):
        # a NZB Logger set to False uses stderr
        script = ScriptBase(logger=False, debug=VERY_VERBOSE_DEBUG)

        # An invalid regular expression (bracket has no close)
        results = script.parse_regex('(.*\.mkv, ', simple=False)
        assert(len(results) == 0)

        results = script.parse_regex('.*\.mkv, .*\.avi,', simple=False)
        assert(len(results) == 2)

        # Now we'll scan our match and we should be okay
        assert(next((True for x in results
                     if x.match('test.mkv')), False) is True)

        # We're case sensitive too!
        assert(next((True for x in results
                     if x.match('test.MKV')), False) is True)

        assert(next((True for x in results
                     if x.match('test.aVi')), False) is True)

        # We don't match against other things not specified in our regular
        # expression
        assert(next((True for x in results
                     if x.match('test.iso')), False) is False)

        # The simple mode (is the default) and allows us to create our
        # regular expression easier using the old linux/windows commands
        # like ?? and *
        results = script.parse_regex('*.mkv, *.avi')
        assert(len(results) == 2)



from setuptools import setup, find_packages

setup(
    name='pynzbget',
    version='0.6.2',
    description="Provides a framework for NZBGet script deployment and development",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
    keywords='nzbget,framework,scripts,nzb',
    author='Chris Caron',
    author_email='lead2gold@gmail.com',
    url='http://github.com/caronc/pynzbget',
    license='GPLv3',
    include_package_data=True,
    zip_safe=False,
    install_requires=['setuptools'],
    requires=['lxml', 'sqlite3'],
    packages=find_packages(),
    python_requires='>=2.6, <3',
)



# -*- encoding: utf-8 -*-
#
# A scripting wrapper for NZBGet's Scheduler Scripting
#
# Copyright (C) 2014 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
"""
This class was intended to make writing NZBGet Scripts easier to manage and
write by handling the common error handling and provide the most reused code
in a re-usable container. It was initially written to work with NZBGet v13
but provides most backwards compatibility.

It was designed to be inheritied as a base class requiring you to only write
the main() function which should preform the task you are intending.

It looks after fetching all of the environment variables and will parse
the meta information out of the NZB-File.

It allows you to set variables that other scripts can access if they need to
using the set() and get() variables. This is done through a simply self
maintained hash table type structure within a sqlite database. All the
wrapper functions are already written.  If you call 'set('MYKEY', 1')
you can call get('MYKEY') in another script and continue working

push() functions written to pass information back to nzbget using it's
processing engine.

all exceptions are now automatically handled and logging can be easily
changed from stdout, to stderr or to a file.

Test suite built in (using python-nose) to ensure old global variables
will still work as well as make them easier to access and manipulate.

Some inline documentation was based on content provided at:
   - http://nzbget.net/Extension_scripts


############################################################################
Schedule Script Usage/Example
############################################################################

############################################################################
### NZBGET SCHEDULER SCRIPT                                               ###
#
# Describe your Schedule Script here
# Author: Chris Caron <lead2gold@gmail.com>
#

############################################################################
### OPTIONS                                                              ###

#
# Enable NZBGet debug logging (yes, no)
# Debug=no
#

### NZBGET SCHEDULER SCRIPT                                              ###
############################################################################

from nzbget import SchedulerScript

# Now define your class while inheriting the rest
class MySchedulerScript(SchedulerScript):
    def main(self, *args, **kwargs):

        # Version Checking, Environment Variables Present, etc
        if not self.validate():
            # No need to document a failure, validate will do that
            # on the reason it failed anyway
            return False

        # write all of your code here you would have otherwise put in the
        # script

        # All system environment variables (NZBOP_.*) as well as Post
        # Process script specific content (NZBSP_.*)
        # following dictionary (without the NZBOP_ or NZBSP_ prefix):
        print('TEMPDIR (directory is: %s' % self.get('TEMPDIR'))
        print('DESTDIR %s' self.get('DESTDIR'))

        # Set any variable you want by any key.  Note that if you use
        # keys that were defined by the system (such as CATEGORY, DIRECTORY,
        # etc, you may have some undesirable results.  Try to avoid reusing
        # system variables already defined (identified above):
        self.set('MY_KEY', 'MY_VALUE')

        # You can fetch it back; this will also set an entry in  the
        # sqlite database for each hash references that can be pulled from
        # another script that simply calls self.get('MY_KEY')
        print(self.get('MY_KEY')) # prints MY_VALUE

        # You can also use push() which is similar to set()
        # except that it interacts with the NZBGet Server and does not use
        # the sqlite database. This can only be reached across other
        # scripts if the calling application is NZBGet itself
        self.push('ANOTHER_KEY', 'ANOTHER_VALUE')

        # You can still however locally retrieve what you set using push()
        # with the get() function
        print(self.get('ANOTHER_KEY')) # prints ANOTHER_VALUE

        # Your script configuration files (NZBNP_.*) are here in this
        # dictionary (again without the NZBNP_ prefix):
        # assume you defined `Debug=no` in the first 10K of your SchedulerScript
        # NZBGet translates this to `NZBNP_DEBUG` which can be retrieved
        # as follows:
        print('DEBUG %s' self.get('DEBUG'))

        # Returns have been made easy.  Just return:
        #   * True if everything was successful
        #   * False if there was a problem
        #   * None if you want to report that you've just gracefully
                  skipped processing (this is better then False)
                  in some circumstances. This is neither a failure or a
                  success status.

        # Feel free to use the actual exit codes as well defined by
        # NZBGet on their website.  They have also been defined here
        # from nzbget.ScriptBase import EXIT_CODE

        return True

# Call your script as follows:
if __name__ == "__main__":
    from sys import exit

    # Create an instance of your Script
    myscript = MySchedulerScript()

    # call run() and exit() using it's returned value
    exit(myscript.run())

"""
import re
from os import environ

# Relative Includes
from .ScriptBase import ScriptBase
from .ScriptBase import NZBGET_BOOL_FALSE
from .ScriptBase import SCRIPT_MODE

# Environment variable that prefixes all NZBGET options being passed into
# scripts with respect to the NZB-File (used in Scan Scripts)
SCHEDULER_ENVIRO_ID = 'NZBSP_'
TASK_ENVIRO_ID = 'TASK'

# Precompile Regulare Expression for Speed
SCHEDULER_OPTS_RE = re.compile('^%s([A-Z0-9_]+)$' % SCHEDULER_ENVIRO_ID)

class SchedulerScript(ScriptBase):
    def __init__(self, *args, **kwargs):
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Multi-Script Support
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        if not hasattr(self, 'script_dict'):
            # Only define once
            self.script_dict = {}
        self.script_dict[SCRIPT_MODE.SCHEDULER] = self

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Initialize Parent
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        super(SchedulerScript, self).__init__(*args, **kwargs)

    def scheduler_init(self, *args, **kwargs):
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Fetch Script Specific Arguments
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        taskid = kwargs.get('taskid')

        # Fetch/Load Scan Script Configuration
        script_config = dict([(SCHEDULER_OPTS_RE.match(k).group(1), v.strip()) \
               for (k, v) in environ.items() if SCHEDULER_OPTS_RE.match(k)])

        if self.vvdebug:
            # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
            # Print Global Script Varables to help debugging process
            # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
            for k, v in script_config.items():
                self.logger.vvdebug('%s%s=%s' % (SCHEDULER_ENVIRO_ID, k, v))

        # Merge Script Configuration With System Config
        self.system = dict(script_config.items() + self.system.items())

        # self.taskid
        # This is the Task Identifier passed in from NZBGet
        if taskid is None:
            self.taskid = environ.get(
                '%sTASKID' % SCHEDULER_ENVIRO_ID,
            )
        else:
            self.taskid = taskid

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Error Handling
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        try:
            self.taskid = int(self.taskid)
            self.logger.info('Task ID assigned: %d' % self.taskid)
        except (ValueError, TypeError):
            # Default is 0
            self.taskid = 0
            self.logger.warning('No Task ID was assigned')

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Enforce system/global variables for script processing
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        self.system['TASKID'] = self.taskid
        if isinstance(self.taskid, int) and self.taskid > 0:
            environ['%sTASKID' % SCHEDULER_ENVIRO_ID] = str(self.taskid)

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Debug Flag Check
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def scheduler_debug(self, *args, **kwargs):
        """Uses the environment variables to detect if debug mode is set
        """
        return self.parse_bool(
            environ.get('%sDEBUG' % SCHEDULER_ENVIRO_ID, NZBGET_BOOL_FALSE),
        )

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Validatation
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def scheduler_validate(self, keys=None, min_version=11, *args, **kargs):
        """validate against environment variables
        """
        is_okay = super(SchedulerScript, self)._validate(
            keys=keys,
            min_version=min_version,
        )

        required_opts = set((
            'TASKID',
        ))

        found_opts = set(self.system) & required_opts
        if found_opts != required_opts:
            missing_opts = list(required_opts ^ found_opts)
            self.logger.error(
                'Validation - (v11) Directives not set: %s' % \
                  missing_opts.join(', ')
            )
            is_okay = False

        return is_okay

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Sanity
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def scheduler_sanity_check(self, *args, **kargs):
        """Sanity checking to ensure this really is a post_process script
        """
        from PostProcessScript import POSTPROC_ENVIRO_ID
        return ('%sDIRECTORY' % POSTPROC_ENVIRO_ID not in environ) and \
               ('%sTASKID' % SCHEDULER_ENVIRO_ID in environ)

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Tasks
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def get_task(self, taskid=None):
        """Returns a dictionary of task details identified by the id
        specified.  If no id is specified, then the current task is
        detected and returned.
        """
        if taskid is None:
            # assume default
            taskid = self.taskid

        if not isinstance(taskid, int):
            try:
                taskid = int(taskid)
            except (ValueError, TypeError):
                # can't be typecasted to an integer
                return {}

        if taskid <= 0:
            # No task defined
            return {}

        # Precompile Regulare Expression for Speed
        task_re = re.compile('^%s%s%d_([A-Z0-9_]+)$' % (
            SCHEDULER_ENVIRO_ID,
            TASK_ENVIRO_ID,
            taskid,
        ))

        self.logger.debug('Looking for %s%s%d_([A-Z0-9_]+)$' % (
            SCHEDULER_ENVIRO_ID,
            TASK_ENVIRO_ID,
            taskid,
        ))

        # Fetch Task related content
        return dict([(task_re.match(k).group(1), v.strip()) \
            for (k, v) in environ.items() if task_re.match(k)])



# -*- encoding: utf-8 -*-
#
# A scripting wrapper for NZBGet Post and Pre Processing
#
# Copyright (C) 2014-2017 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
"""
This class simplifies the setup of the logging to screen or file. It was
written to simplify the logging to and from an NZBGet Script
"""
import sys
import logging
import logging.handlers
from logging import Logger

from os import getpid

# Monkey Patch
logging.raiseExceptions = 0

# Logging Levels
DETAIL = 19
DEBUG = logging.DEBUG
VERBOSE_DEBUG = logging.DEBUG-1
VERY_VERBOSE_DEBUG = logging.DEBUG-2

# Ensure Levels are Globally Added To logging module
logging.addLevelName(DETAIL, "DETAIL")
logging.addLevelName(VERBOSE_DEBUG, "VDEBUG")
logging.addLevelName(VERY_VERBOSE_DEBUG, "VVDEBUG")

def detail(self, message, *args, **kwargs):
    # logger takes its '*args' as 'args'.
    self._log(DETAIL, message, args, **kwargs)

def vdebug(self, message, *args, **kwargs):
    # logger takes its '*args' as 'args'.
    self._log(VERBOSE_DEBUG, message, args, **kwargs)

def vvdebug(self, message, *args, **kwargs):
    # logger takes its '*args' as 'args'.
    self._log(VERY_VERBOSE_DEBUG, message, args, **kwargs)

logging.Logger.detail = detail
logging.Logger.vdebug = vdebug
logging.Logger.vvdebug = vvdebug

def destroy_logger(name=None):
    """
       Destroys any log files assiated with name and/or logger
    """
    if name is None:
        name = __name__

    if isinstance(name, Logger):
        logger = name
    elif isinstance(name, basestring):
        logger = logging.getLogger(name)
    else:
        # not supported
        return

    logger = name
    if hasattr(logger, 'handlers'):
        # Destroy Logger
        for l in logger.handlers:
            logger.removeHandler(l)
            l.flush()
            try:
                l.close()
            except KeyError:
                # https://bugzilla.redhat.com/show_bug.cgi?id=573782
                # Bug 573782 - python logging's fileConfig causes
                # KeyError on shutdown
                pass

def init_logger(name=None, logger=True, debug=False, nzbget_mode=True,
                daily=False, bytecount=5242880, logcount=3, encoding=None):
    """
        Generate Logger

        name:         Defines a name to help identify each log entry
        logger:       - If set to 'None', then logging is disabled.
                      - If set to 'True', then content is sent to stdout
                        is used.
                      - If set to 'False', then content is sent to
                        stderr
                      - If set to a string, then the string is presumed
                        to be the filename where the logging should be
                        sent.
                      - If defined as a logger, then that is presumed
                        to be the log file to use.
        debug:        Enable extra debug log entries, if this is an interger
                      value, then that is the log level set
        nzbget_mode:  Log formatting does not include date/time, pid etc
                      because nzbget wraps all this for us.
        daily:        Rotate logs by day (instead of by size)
        encoding:     Can be set to something like 'bz2' to have all
                      content created compressed
    """

    if isinstance(logger, Logger):
        # Update handlers only
        for l in logger.handlers:
            if not nzbget_mode:
                l.setFormatter(logging. \
                    Formatter("%(asctime)s - " + str(getpid()) + \
                        " - %(levelname)s - %(message)s"))
            else:
                l.setFormatter(logging. \
                    Formatter("[%(levelname)s] %(message)s"))

        # Support NZBGET Detail Messages
        logging.addLevelName(DETAIL, 'DETAIL')

        if not nzbget_mode:
            logging.addLevelName(logging.DEBUG, 'DEBUG')
            logging.addLevelName(VERBOSE_DEBUG, 'VDEBUG')
            logging.addLevelName(VERY_VERBOSE_DEBUG, 'VVDEBUG')

        else:
            # Level Name for [debug] has to be [info] or it simply won't print
            logging.addLevelName(logging.DEBUG, 'INFO] [DEBUG')
            logging.addLevelName(VERBOSE_DEBUG, 'INFO] [VDEBUG')
            logging.addLevelName(VERY_VERBOSE_DEBUG, 'INFO] [VVDEBUG')

        return logger

    if name is None:
        # Namespace
        name = __name__

    # Perpare Logger
    _logger = logging.getLogger(name)

    # Ensure the logger isn't already initialized
    # to limit the number of handlers to 1, we sweep anything
    # that may or may not have already been created
    if isinstance(_logger, Logger):
        for l in _logger.handlers:
            _logger.removeHandler(l)
            l.flush()
            l.close()

    if logger is None:
        # Create a dummy log file without a handler
        _logger.setLevel(logging.CRITICAL)
        #logging.disable(logging.ERROR)
        # no logging handler nessisary
        return _logger

    elif isinstance(logger, basestring):
        # prepare rotating handler using the log file specified
        if not daily:
            h1 = logging.handlers.RotatingFileHandler(
                logger,
                maxBytes=bytecount,
                backupCount=logcount,
                encoding=encoding,
            )
        else:
            h1 = logging.handlers.TimedRotatingFileHandler(
                 logger,
                 when='midnight',
                 interval=1,
                 backupCount=logcount,
                 encoding=encoding,
            )

    elif logger:
        # stdout
        h1 = logging.StreamHandler(sys.stdout)
    else:
        # stderr
        h1 = logging.StreamHandler(sys.stderr)

    if debug is True:
        _logger.setLevel(logging.DEBUG)
        h1.setLevel(logging.DEBUG)

    elif debug in (False, None):
        # Default
        _logger.setLevel(DETAIL)
        h1.setLevel(DETAIL)
    else:
        try:
            debug = int(debug)
            _logger.setLevel(debug)
            h1.setLevel(debug)

        except (ValueError, TypeError):
            # Default
            _logger.setLevel(DETAIL)
            h1.setLevel(DETAIL)

    # Format logger
    if not nzbget_mode:
        h1.setFormatter(logging. \
                Formatter("%(asctime)s - " + str(getpid()) +
                    " - %(levelname)s - %(message)s"))
        logging.addLevelName(logging.DEBUG, 'DEBUG')
        logging.addLevelName(VERBOSE_DEBUG, 'VDEBUG')
        logging.addLevelName(VERY_VERBOSE_DEBUG, 'VVDEBUG')

    else:
        h1.setFormatter(logging. \
                Formatter("[%(levelname)s] %(message)s"))
        # Level Name for [debug] has to be [info] or it simply won't print
        logging.addLevelName(logging.DEBUG, 'INFO] [DEBUG')
        logging.addLevelName(VERBOSE_DEBUG, 'INFO] [VDEBUG')
        logging.addLevelName(VERY_VERBOSE_DEBUG, 'INFO] [VVDEBUG')

    # Add Handler
    _logger.addHandler(h1)

    return _logger



# -*- encoding: utf-8 -*-
#
# A Test Suite (for nose) for the PostProcessScript Class
#
# Copyright (C) 2014-2017 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
import os
import sys
import re
from os import makedirs
from os.path import basename
from os.path import join
from os.path import splitext

from TestBase import TestBase
from TestBase import TEMP_DIRECTORY

from nzbget.ScriptBase import CFG_ENVIRO_ID
from nzbget.ScriptBase import SYS_ENVIRO_ID
from nzbget.ScriptBase import PUSH_ENVIRO_ID
from nzbget.ScriptBase import NZBGET_MSG_PREFIX
from nzbget.ScriptBase import SHR_ENVIRO_GUESS_ID
from nzbget.ScriptBase import EXIT_CODE
from nzbget.ScriptBase import Health

from nzbget.PostProcessScript import PostProcessScript
from nzbget.PostProcessScript import POSTPROC_ENVIRO_ID
from nzbget.PostProcessScript import SCRIPT_STATUS
from nzbget.PostProcessScript import TOTAL_STATUS
from nzbget.PostProcessScript import PAR_STATUS
from nzbget.PostProcessScript import UNPACK_STATUS

from nzbget.Logger import VERY_VERBOSE_DEBUG


import StringIO

from shutil import rmtree

# Some constants to work with
DIRECTORY = TEMP_DIRECTORY
NZBNAME = 'A.Great.Movie'
NZBFILENAME = join(TEMP_DIRECTORY, 'A.Great.Movie.nzb')
NZBFILENAME_SHOW_A = join(TEMP_DIRECTORY, 'A.Great.TV.Show.nzb')
NZBFILENAME_SHOW_B = join(TEMP_DIRECTORY, 'Another.Great.TV.Show.nzb')
CATEGORY = 'movie'
TOTALSTATUS = TOTAL_STATUS.SUCCESS
SCRIPTSTATUS = SCRIPT_STATUS.SUCCESS
STATUS = 'SUCCESS/ALL'
PARSTATUS = PAR_STATUS.SUCCESS
UNPACKSTATUS = UNPACK_STATUS.SUCCESS
VERSION = 13

SEARCH_DIR = join(TEMP_DIRECTORY, 'file_listing')

# For validation
SCRIPTDIR = join(TEMP_DIRECTORY, 'scripts')


class TestPostProcessScript(TestBase):
    def setUp(self):
        """This method is run once before _each_ test method is executed"""
        super(TestPostProcessScript, self).setUp()

        # Create NZBFILE
        _f = open(NZBFILENAME, 'w')
        _f.write("""<?xml version="1.0" encoding="UTF-8"?>
            <!DOCTYPE nzb PUBLIC "-//newzBin//DTD NZB 1.1//EN" "http://www.newzbin.com/DTD/nzb/nzb-1.1.dtd">
            <nzb xmlns="http://www.newzbin.com/DTD/2003/nzb">

            <head>
             <meta type="category">Movies &gt; SD</meta>
             <meta type="name">A.Great.Movie.1983.DVDRip.x264-AWESOME</meta>
             <meta type="propername">A Great Movie</meta>
             <meta type="movieyear">1983</meta>
            <!-- extra record used to determine what file we're reading -->
             <meta type="letter">A</meta>
            </head>
          </nzb>""")
        _f.close()

        _f = open(NZBFILENAME_SHOW_A, 'w')
        _f.write("""<?xml version="1.0" encoding="UTF-8"?>
            <!DOCTYPE nzb PUBLIC "-//newzBin//DTD NZB 1.1//EN" "http://www.newzbin.com/DTD/nzb/nzb-1.1.dtd">
            <nzb xmlns="http://www.newzbin.com/DTD/2003/nzb">

            <head>
             <meta type="category">TV &gt; HD</meta>
             <meta type="name">A.Great.TV.Show.S04E06.720p.HDTV.x264-AWESOME</meta>
             <meta type="propername">A Great TV Show</meta>
             <meta type="episodename">An Amazing Episode Name</meta>
            <!-- extra record used to determine what file we're reading -->
             <meta type="letter">B</meta>
            </head>
          </nzb>""")
        _f.close()

        _f = open('%s.queued' % NZBFILENAME_SHOW_A, 'w')
        _f.write("""<?xml version="1.0" encoding="UTF-8"?>
            <!DOCTYPE nzb PUBLIC "-//newzBin//DTD NZB 1.1//EN" "http://www.newzbin.com/DTD/nzb/nzb-1.1.dtd">
            <nzb xmlns="http://www.newzbin.com/DTD/2003/nzb">

            <head>
             <meta type="category">TV &gt; HD</meta>
             <meta type="name">A.Great.TV.Show.S04E06.720p.HDTV.x264-AWESOME</meta>
             <meta type="propername">A Great TV Show</meta>
             <meta type="episodename">An Amazing Episode Name</meta>
            <!-- extra record used to determine what file we're reading -->
             <meta type="letter">C</meta>
            </head>
          </nzb>""")
        _f.close()

        _f = open('%s' % NZBFILENAME_SHOW_B, 'w')
        _f.write("""<?xml version="1.0" encoding="UTF-8"?>
            <!DOCTYPE nzb PUBLIC "-//newzBin//DTD NZB 1.1//EN" "http://www.newzbin.com/DTD/nzb/nzb-1.1.dtd">
            <nzb xmlns="http://www.newzbin.com/DTD/2003/nzb">

            <head>
             <meta type="category">TV &gt; HD</meta>
             <meta type="name">Another.Great.TV.Show.S01E02.720p.HDTV.x264-AWESOME</meta>
             <meta type="propername">Another Great TV Show</meta>
             <meta type="episodename">An Okay Episode Name</meta>
            <!-- extra record used to determine what file we're reading -->
             <meta type="letter">D</meta>
            </head>
          </nzb>""")
        _f.close()

        _f = open('%s.queued' % NZBFILENAME_SHOW_B, 'w')
        _f.write("""<?xml version="1.0" encoding="UTF-8"?>
            <!DOCTYPE nzb PUBLIC "-//newzBin//DTD NZB 1.1//EN" "http://www.newzbin.com/DTD/nzb/nzb-1.1.dtd">
            <nzb xmlns="http://www.newzbin.com/DTD/2003/nzb">

            <head>
             <meta type="category">TV &gt; HD</meta>
             <meta type="name">Another.Great.TV.Show.S01E02.720p.HDTV.x264-AWESOME</meta>
             <meta type="propername">Another Great TV Show</meta>
             <meta type="episodename">An Okay Episode Name</meta>
            <!-- extra record used to determine what file we're reading -->
             <meta type="letter">E</meta>
            </head>
          </nzb>""")
        _f.close()

        _f = open('%s.2.queued' % NZBFILENAME_SHOW_B, 'w')
        _f.write("""<?xml version="1.0" encoding="UTF-8"?>
            <!DOCTYPE nzb PUBLIC "-//newzBin//DTD NZB 1.1//EN" "http://www.newzbin.com/DTD/nzb/nzb-1.1.dtd">
            <nzb xmlns="http://www.newzbin.com/DTD/2003/nzb">

            <head>
             <meta type="category">TV &gt; HD</meta>
             <meta type="name">Another.Great.TV.Show.S01E02.720p.HDTV.x264-AWESOME</meta>
             <meta type="propername">Another Great TV Show</meta>
             <meta type="episodename">An Okay Episode Name</meta>
            <!-- extra record used to determine what file we're reading -->
             <meta type="letter">F</meta>
            </head>
          </nzb>""")
        _f.close()

        # Create some environment variables
        os.environ['%sSCRIPTDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sTEMPDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sVERSION' % SYS_ENVIRO_ID] = str(VERSION)
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = DIRECTORY
        os.environ['%sNZBNAME' % POSTPROC_ENVIRO_ID] = NZBNAME
        os.environ['%sNZBFILENAME' % POSTPROC_ENVIRO_ID] = NZBFILENAME
        os.environ['%sCATEGORY' % POSTPROC_ENVIRO_ID] = CATEGORY
        os.environ['%sTOTALSTATUS' % POSTPROC_ENVIRO_ID] = TOTALSTATUS
        os.environ['%sSTATUS' % POSTPROC_ENVIRO_ID] = STATUS
        os.environ['%sSCRIPTSTATUS' % POSTPROC_ENVIRO_ID] = SCRIPTSTATUS
        # NZBGet v11 support
        os.environ['%sPARSTATUS' % POSTPROC_ENVIRO_ID] = str(PARSTATUS)
        os.environ['%sUNPACKSTATUS' % POSTPROC_ENVIRO_ID] = str(UNPACKSTATUS)

        # ensure directory doesn't exist
        try:
            rmtree(SEARCH_DIR)
        except:
            pass
        makedirs(SEARCH_DIR)

    def tearDown(self):
        """This method is run once after _each_ test method is executed"""
        # Eliminate any variables defined
        if '%sSCRIPTDIR' % SYS_ENVIRO_ID in os.environ:
            del os.environ['%sSCRIPTDIR' % SYS_ENVIRO_ID]
        del os.environ['%sTEMPDIR' % SYS_ENVIRO_ID]
        del os.environ['%sVERSION' % SYS_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]
        del os.environ['%sNZBNAME' % POSTPROC_ENVIRO_ID]
        del os.environ['%sNZBFILENAME' % POSTPROC_ENVIRO_ID]
        del os.environ['%sCATEGORY' % POSTPROC_ENVIRO_ID]
        del os.environ['%sTOTALSTATUS' % POSTPROC_ENVIRO_ID]
        del os.environ['%sSCRIPTSTATUS' % POSTPROC_ENVIRO_ID]
        del os.environ['%sPARSTATUS' % POSTPROC_ENVIRO_ID]
        del os.environ['%sUNPACKSTATUS' % POSTPROC_ENVIRO_ID]

        try:
            rmtree(SEARCH_DIR)
        except:
            pass

        # common
        super(TestPostProcessScript, self).tearDown()

    def test_main_returns(self):
        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.run() == EXIT_CODE.SUCCESS

        class TestPostProcessMain(PostProcessScript):
            def postprocess_main(self, *args, **kwargs):
                return None

        script = TestPostProcessMain(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.run() == EXIT_CODE.NONE

        del script
        del os.environ['%sSCRIPTDIR' % SYS_ENVIRO_ID]
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.run() == EXIT_CODE.FAILURE

    def test_environment_varable_init(self):
        """
        Testing NZBGet Script initialization using environment variables
        """
        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.directory == DIRECTORY
        assert script.nzbname == NZBNAME
        assert script.nzbfilename == NZBFILENAME
        assert script.category == CATEGORY
        assert script.totalstatus == TOTALSTATUS
        assert str(script.status) == STATUS
        assert script.scriptstatus == SCRIPTSTATUS
        assert script.parstatus == PARSTATUS
        assert script.unpackstatus == UNPACKSTATUS

        assert script.system['TEMPDIR'] == TEMP_DIRECTORY
        assert script.system['VERSION'] == str(VERSION)
        assert script.system['DIRECTORY'] == DIRECTORY
        assert script.system['NZBNAME'] == NZBNAME
        assert script.system['NZBFILENAME'] == NZBFILENAME
        assert script.system['CATEGORY'] == CATEGORY
        assert script.system['TOTALSTATUS'] == TOTALSTATUS
        assert script.system['STATUS'] == STATUS
        assert script.system['SCRIPTSTATUS'] == SCRIPTSTATUS
        assert script.system['PARSTATUS'] == PARSTATUS
        assert script.system['UNPACKSTATUS'] == UNPACKSTATUS

        assert script.get('TEMPDIR') == TEMP_DIRECTORY
        assert script.get('VERSION') == str(VERSION)
        assert script.get('DIRECTORY') == DIRECTORY
        assert script.get('NZBNAME') == NZBNAME
        assert script.get('NZBFILENAME') == NZBFILENAME
        assert script.get('CATEGORY') == CATEGORY
        assert script.get('TOTALSTATUS') == TOTALSTATUS
        assert script.get('STATUS') == STATUS
        assert script.get('SCRIPTSTATUS') == SCRIPTSTATUS
        assert script.get('PARSTATUS') == PARSTATUS
        assert script.get('UNPACKSTATUS') == UNPACKSTATUS

        assert len(script.config) == 1
        assert script.config.get('DEBUG') == VERY_VERBOSE_DEBUG

        assert script.nzbheaders['MOVIEYEAR'] == '1983'
        assert script.nzbheaders['NAME'] == \
            'A.Great.Movie.1983.DVDRip.x264-AWESOME'
        assert script.nzbheaders['PROPERNAME'] == 'A Great Movie'
        assert script.nzbheaders['CATEGORY'] == 'Movies > SD'

        assert script.nzb_get('movieyear') == '1983'
        assert script.nzb_get('name') == \
            'A.Great.Movie.1983.DVDRip.x264-AWESOME'
        assert script.nzb_get('propername') == 'A Great Movie'
        assert script.nzb_get('category') == 'Movies > SD'

        assert os.environ['%sTEMPDIR' % SYS_ENVIRO_ID] == TEMP_DIRECTORY
        assert os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] == DIRECTORY
        assert os.environ['%sNZBNAME' % POSTPROC_ENVIRO_ID] == NZBNAME
        assert os.environ['%sNZBFILENAME' % POSTPROC_ENVIRO_ID] == NZBFILENAME
        assert os.environ['%sCATEGORY' % POSTPROC_ENVIRO_ID] == CATEGORY
        assert os.environ['%sTOTALSTATUS' % POSTPROC_ENVIRO_ID] == TOTALSTATUS
        assert os.environ['%sSTATUS' % POSTPROC_ENVIRO_ID] == STATUS
        assert os.environ['%sSCRIPTSTATUS' % POSTPROC_ENVIRO_ID] == \
            SCRIPTSTATUS
        assert os.environ['%sPARSTATUS' % POSTPROC_ENVIRO_ID] == str(PARSTATUS)
        assert os.environ['%sUNPACKSTATUS' % POSTPROC_ENVIRO_ID] == \
            str(UNPACKSTATUS)

    def test_environment_override(self):
        """
        Testing NZBGet Script initialization using forced variables that
        should take priority over global ones
        """
        directory = join(DIRECTORY, 'a', 'deeper', 'path')
        # ensure directory doesn't exist
        try:
            rmtree(directory)
        except:
            pass
        makedirs(directory)

        nzbname = '%s.with.more.content' % NZBNAME
        nzbfilename = join(directory, basename(NZBFILENAME))
        category = '%s2' % CATEGORY
        totalstatus = TOTAL_STATUS.DELETED
        scriptstatus = SCRIPT_STATUS.FAILURE
        status = 'FAILURE/UNPACK'
        parstatus = PAR_STATUS.FAILURE
        unpackstatus = UNPACK_STATUS.FAILURE

        script = PostProcessScript(
            logger=False,
            debug=VERY_VERBOSE_DEBUG,

            directory=directory,
            nzbname=nzbname,
            nzbfilename=nzbfilename,
            category=category,
            totalstatus=totalstatus,
            status=status,
            scriptstatus=scriptstatus,

            # v11 Support
            parstatus=parstatus,
            unpackstatus=unpackstatus,
        )

        assert script.directory == directory
        assert script.nzbname == nzbname
        assert script.nzbfilename == nzbfilename
        assert script.category == category
        assert script.totalstatus == totalstatus
        assert str(script.status) == status
        assert script.scriptstatus == scriptstatus
        assert script.parstatus == parstatus
        assert script.unpackstatus == unpackstatus

        assert script.system['TEMPDIR'] == TEMP_DIRECTORY
        assert script.system['DIRECTORY'] == directory
        assert script.system['NZBNAME'] == nzbname
        assert script.system['NZBFILENAME'] == nzbfilename
        assert script.system['CATEGORY'] == category
        assert script.system['TOTALSTATUS'] == totalstatus
        assert script.system['STATUS'] == status
        assert script.system['SCRIPTSTATUS'] == scriptstatus
        assert script.system['PARSTATUS'] == parstatus
        assert script.system['UNPACKSTATUS'] == unpackstatus

        assert len(script.config) == 1
        assert script.config.get('DEBUG') == VERY_VERBOSE_DEBUG
        assert script.shared == {}
        assert script.nzbheaders == {}

        assert os.environ['%sTEMPDIR' % SYS_ENVIRO_ID] == TEMP_DIRECTORY
        assert os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] == directory
        assert os.environ['%sNZBNAME' % POSTPROC_ENVIRO_ID] == nzbname
        assert os.environ['%sNZBFILENAME' % POSTPROC_ENVIRO_ID] == nzbfilename
        assert os.environ['%sCATEGORY' % POSTPROC_ENVIRO_ID] == category
        assert os.environ['%sTOTALSTATUS' % POSTPROC_ENVIRO_ID] == totalstatus
        assert os.environ['%sSTATUS' % POSTPROC_ENVIRO_ID] == status
        assert os.environ['%sSCRIPTSTATUS' % POSTPROC_ENVIRO_ID] == \
            scriptstatus
        assert os.environ['%sPARSTATUS' % POSTPROC_ENVIRO_ID] == \
            str(parstatus)
        assert os.environ['%sUNPACKSTATUS' % POSTPROC_ENVIRO_ID] == \
            str(unpackstatus)

        # cleanup
        try:
            rmtree(directory)
        except:
            pass

    def test_set_and_get(self):
        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)

        KEY = 'MY_VAR'
        VALUE = 'MY_VALUE'

        # Value doe snot exist yet
        assert script.get(KEY) is None
        assert script.get(KEY, 'Default') == 'Default'
        assert script.set(KEY, VALUE) is True
        assert script.get(KEY, 'Default') == VALUE

    def test_config_varable_init(self):
        valid_entries = {
            'MY_CONFIG_ENTRY': 'Option A',
            'ENTRY_WITH_234_NUMBERS': 'Option B',
            '123443': 'Option C',
        }
        invalid_entries = {
            'CONFIG_ENtry_skipped': 'Option',
            'CONFIG_ENtry_$#': 'Option',
            # Empty
            '': 'Option',
        }
        for k, v in valid_entries.items():
            os.environ['%s%s' % (CFG_ENVIRO_ID, k)] = v
        for k, v in invalid_entries.items():
            os.environ['%s%s' % (CFG_ENVIRO_ID, k)] = v

        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        for k, v in valid_entries.items():
            assert k in script.config
            assert script.config[k] == v

        for k in invalid_entries.keys():
            assert k not in script.config

        # Cleanup
        for k, v in valid_entries.items():
            del os.environ['%s%s' % (CFG_ENVIRO_ID, k)]
        for k, v in invalid_entries.items():
            del os.environ['%s%s' % (CFG_ENVIRO_ID, k)]

    def test_nzbget_push_directory(self):

        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)

        directory = join(DIRECTORY, 'new', 'path')

        # Keep a handle on the real standard output
        stdout = sys.stdout
        sys.stdout = StringIO.StringIO()
        script.push_directory(directory)

        # extract data
        sys.stdout.seek(0)
        output = sys.stdout.read().strip()

        # return stdout back to how it was
        sys.stdout = stdout
        assert output == '%s%s=%s' % (
            NZBGET_MSG_PREFIX,
            'DIRECTORY',
            directory,
        )
        assert script.directory == directory
        assert script.system['DIRECTORY'] == directory

    def test_nzbget_push_final_directory(self):

        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)

        directory = join(DIRECTORY, 'new', 'path')

        # Keep a handle on the real standard output
        stdout = sys.stdout
        sys.stdout = StringIO.StringIO()
        script.push_directory(directory, final=True)

        # extract data
        sys.stdout.seek(0)
        output = sys.stdout.read().strip()

        # return stdout back to how it was
        sys.stdout = stdout
        assert output == '%s%s=%s' % (
            NZBGET_MSG_PREFIX,
            'FINALDIR',
            directory,
        )
        assert script.directory == directory
        assert script.system['DIRECTORY'] == directory

    def test_pushes(self):
        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)

        KEY = 'MY_VAR'
        VALUE = 'MY_VALUE'

        # Value doe snot exist yet
        assert script.get(KEY) is None

        # Keep a handle on the real standard output
        stdout = sys.stdout
        sys.stdout = StringIO.StringIO()
        script.push(KEY, VALUE)

        # extract data
        sys.stdout.seek(0)
        output = sys.stdout.read().strip()

        # return stdout back to how it was
        sys.stdout = stdout
        assert output == '%s%s%s=%s' % (
            NZBGET_MSG_PREFIX,
            PUSH_ENVIRO_ID,
            KEY,
            VALUE,
        )
        assert script.get(KEY) == VALUE

    def test_validation(self):

        # This will fail because it's looking for the SCRIPTDIR
        # variable defined with NZBGet v11
        # a NZB Logger set to False uses stderr
        if '%sSCRIPTDIR' % SYS_ENVIRO_ID in os.environ:
            del os.environ['%sSCRIPTDIR' % SYS_ENVIRO_ID]
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert not script.validate()
        del script

        # Now let's set it and try again
        os.environ['%sSCRIPTDIR' % SYS_ENVIRO_ID] = SCRIPTDIR
        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.validate()
        del os.environ['%sSCRIPTDIR' % SYS_ENVIRO_ID]
        del script

        # Now let's set it and try again
        os.environ['%sSCRIPTDIR' % SYS_ENVIRO_ID] = SCRIPTDIR
        os.environ['%sVALUE_A' % CFG_ENVIRO_ID] = 'A'
        os.environ['%sVALUE_B' % CFG_ENVIRO_ID] = 'B'
        os.environ['%sVALUE_C' % CFG_ENVIRO_ID] = 'C'

        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.validate(keys=(
            'Value_A', 'VALUE_B', 'value_c'
        ))
        del os.environ['%sSCRIPTDIR' % SYS_ENVIRO_ID]
        del os.environ['%sVALUE_A' % CFG_ENVIRO_ID]
        del os.environ['%sVALUE_B' % CFG_ENVIRO_ID]
        del os.environ['%sVALUE_C' % CFG_ENVIRO_ID]

    def test_health_check(self):
        """Test that wrapper for healthcheck is working correctly
        """
        # This will fail because it's looking for the SCRIPTDIR
        # variable defined with NZBGet v11
        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.health_check() is True
        assert script.health_check(
            is_unpacked=False, has_archive=False) is True

        assert script.health_check(is_unpacked=False) is True
        assert script.health_check(has_archive=True) is False

        script.version = 11
        assert script.health_check() is True
        assert script.health_check(
            is_unpacked=False, has_archive=False) is True

        assert script.health_check(is_unpacked=False) is True
        assert script.health_check(has_archive=True) is True
        del script

        os.environ['%sSTATUS' % POSTPROC_ENVIRO_ID] = '%s/%s' % (
            Health.FAILURE,
            Health.DEFAULT_SUB,
        )
        # NZBGet v11 support
        os.environ['%sPARSTATUS' % POSTPROC_ENVIRO_ID] = \
            str(PAR_STATUS.FAILURE)
        os.environ['%sUNPACKSTATUS' % POSTPROC_ENVIRO_ID] = \
            str(UNPACK_STATUS.FAILURE)

        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.health_check() is False
        assert script.health_check(
            is_unpacked=False, has_archive=False) is True

        assert script.health_check(is_unpacked=False) is True
        assert script.health_check(has_archive=True) is False

        script.version = 11
        assert script.health_check() is False
        assert script.health_check(
            is_unpacked=False, has_archive=False) is True

        assert script.health_check(is_unpacked=False) is True
        assert script.health_check(has_archive=True) is False

    def test_file_listings_as_string(self):

        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.get_files(search_dir=SEARCH_DIR) == {}

        # Create some temporary files to work with
        open(join(SEARCH_DIR, 'file.mkv'), 'w').close()
        open(join(SEARCH_DIR, 'file.par2'), 'w').close()
        open(join(SEARCH_DIR, 'file.PAR2'), 'w').close()
        open(join(SEARCH_DIR, 'file.txt'), 'w').close()
        open(join(SEARCH_DIR, 'sample.mp4'), 'w').close()
        open(join(SEARCH_DIR, 'sound.mp3'), 'w').close()

        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        files = script.get_files(search_dir=SEARCH_DIR)
        assert len(files) == 6
        assert 'basename' in files[files.keys()[0]]
        assert 'dirname' in files[files.keys()[0]]
        assert 'extension' in files[files.keys()[0]]
        assert 'filesize' not in files[files.keys()[0]]
        assert 'modified' not in files[files.keys()[0]]
        assert 'accessed' not in files[files.keys()[0]]
        assert 'created' not in files[files.keys()[0]]

        files = script.get_files(
            fullstats=True,
            search_dir=SEARCH_DIR,
        )
        assert len(files) == 6
        assert 'basename' in files[files.keys()[0]]
        assert 'dirname' in files[files.keys()[0]]
        assert 'extension' in files[files.keys()[0]]
        assert 'filesize' in files[files.keys()[0]]
        assert 'modified' in files[files.keys()[0]]
        assert 'accessed' in files[files.keys()[0]]
        assert 'created' in files[files.keys()[0]]

        # Test Filters (as strings)
        files = script.get_files(
            regex_filter='.*\.par2$',
            search_dir=SEARCH_DIR,
            case_sensitive=True,
        )
        assert len(files) == 1

        # Test Filters (as strings) (default is case-insensitive)
        files = script.get_files(
            regex_filter='.*\.par2$',
            search_dir=SEARCH_DIR,
        )
        assert len(files) == 2

        # Assume case sensitivity matters to us, we might
        # want to pre-compile our own list and pass it in
        files = script.get_files(
            regex_filter=re.compile('.*\.par2$', re.IGNORECASE),
            search_dir=SEARCH_DIR,
        )
        assert len(files) == 2

        # Test Filters (as strings)
        files = script.get_files(
            suffix_filter='.par2',
            search_dir=SEARCH_DIR,
        )
        assert len(files) == 1

        # Test Filters (as strings)
        # delimiters such as , (comma), / (slash), (space), etc
        # are automatically handled so you can chain them using
        # these
        files = script.get_files(
            suffix_filter='.par2,.PAR2',
            search_dir=SEARCH_DIR,
        )
        assert len(files) == 2

        # Test Filters (as strings)
        files = script.get_files(
            prefix_filter='sound',
            search_dir=SEARCH_DIR,
        )
        assert len(files) == 1

        # Test Filters (as strings)
        files = script.get_files(
            prefix_filter='sound, file',
            search_dir=SEARCH_DIR,
        )
        assert len(files) == 5

        # Test Filters (as strings)
        files = script.get_files(
            prefix_filter='s',
            search_dir=SEARCH_DIR,
        )
        assert len(files) == 2

    def test_file_listings_as_list(self):

        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.get_files(search_dir=[SEARCH_DIR, ]) == {}

        # Create some temporary files to work with
        open(join(SEARCH_DIR, 'file.mkv'), 'w').close()
        open(join(SEARCH_DIR, 'file.par2'), 'w').close()
        open(join(SEARCH_DIR, 'file.PAR2'), 'w').close()
        open(join(SEARCH_DIR, 'file.txt'), 'w').close()
        open(join(SEARCH_DIR, 'sample.mp4'), 'w').close()
        open(join(SEARCH_DIR, 'sound.mp3'), 'w').close()

        # Test Filters
        files = script.get_files(
            search_dir=[SEARCH_DIR, ],
            regex_filter=('.*\.par2$',),
            case_sensitive=True
        )
        assert len(files) == 1

        # Default is not case-sensitive
        files = script.get_files(
            search_dir=[SEARCH_DIR, ],
            regex_filter=('.*\.par2$',),
        )
        assert len(files) == 2

        # Assume case sensitivity matters to us, we might
        # want to pre-compile our own list and pass it in
        files = script.get_files(
            search_dir=[SEARCH_DIR, ],
            regex_filter=(re.compile('.*\.par2$', re.IGNORECASE),),
        )
        assert len(files) == 2

        # Test Filters (as strings)
        files = script.get_files(
            search_dir=[SEARCH_DIR, ],
            suffix_filter=('.par2',),
        )
        assert len(files) == 1

        # Test Filters (as strings)
        # delimiters such as , (comma), / (slash), (space), etc
        # are automatically handled so you can chain them using
        # these
        files = script.get_files(
            search_dir=[SEARCH_DIR, ],
            suffix_filter=('.par2,.PAR2',),
        )
        assert len(files) == 2

        files = script.get_files(
            search_dir=[SEARCH_DIR, ],
            suffix_filter=('.par2', '.PAR2'),
        )
        assert len(files) == 2

        files = script.get_files(
            search_dir=[SEARCH_DIR, ],
            prefix_filter=('sound',),
        )
        assert len(files) == 1

        files = script.get_files(
            search_dir=[SEARCH_DIR, ],
            prefix_filter=('sound, file',),
        )
        assert len(files) == 5

        # Duplicates should merge
        files = script.get_files(
            search_dir=[SEARCH_DIR, SEARCH_DIR, SEARCH_DIR],
            prefix_filter=('sound, file',),
        )
        assert len(files) == 5

    def test_nzbfilename_searching(self):

        # a NZB Logger set to False uses stderr
        script = PostProcessScript(
            logger=False,
            debug=VERY_VERBOSE_DEBUG,
            nzbfilename=NZBFILENAME_SHOW_A
        )
        assert script.parse_nzbfile(NZBFILENAME_SHOW_A)['LETTER'] == 'B'
        os.unlink(NZBFILENAME_SHOW_A)

        script = PostProcessScript(
            logger=False,
            debug=VERY_VERBOSE_DEBUG,
            nzbfilename=NZBFILENAME_SHOW_A
        )
        assert 'LETTER' not in script.parse_nzbfile(NZBFILENAME_SHOW_A)
        assert script.parse_nzbfile(
            NZBFILENAME_SHOW_A, check_queued=True)['LETTER'] == 'C'

        script = PostProcessScript(
            logger=False,
            debug=VERY_VERBOSE_DEBUG,
            nzbfilename=NZBFILENAME_SHOW_B
        )
        assert script.parse_nzbfile(NZBFILENAME_SHOW_B)['LETTER'] == 'D'

    def test_file_obsfucation(self):

        download_dir = join(TEMP_DIRECTORY, 'obsfucation')
        obsfucated_dir = join(download_dir, 'Afjk3af3D')

        # ensure directory doesn't exist
        try:
            rmtree(download_dir)
        except:
            pass
        try:
            rmtree(obsfucated_dir)
        except:
            pass
        makedirs(download_dir)
        makedirs(obsfucated_dir)

        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)

        assert len(script.parse_nzbfile(NZBFILENAME_SHOW_A)) == 5
        assert script.parse_nzbfile(NZBFILENAME_SHOW_A)['LETTER'] == 'B'
        assert script.parse_nzbfile(NZBFILENAME_SHOW_A)['PROPERNAME'] == \
            'A Great TV Show'
        assert script.parse_nzbfile(NZBFILENAME_SHOW_A)['NAME'] == \
            'A.Great.TV.Show.S04E06.720p.HDTV.x264-AWESOME'
        assert script.parse_nzbfile(NZBFILENAME_SHOW_A)['EPISODENAME'] == \
            'An Amazing Episode Name'
        assert script.parse_nzbfile(NZBFILENAME_SHOW_A)['CATEGORY'] == \
            'TV > HD'

        assert len(script.parse_nzbfile(NZBFILENAME)) == 5
        assert script.parse_nzbfile(NZBFILENAME)['LETTER'] == 'A'
        assert script.parse_nzbfile(NZBFILENAME)['CATEGORY'] == \
            'Movies > SD'
        assert script.parse_nzbfile(NZBFILENAME)['PROPERNAME'] == \
            'A Great Movie'
        assert script.parse_nzbfile(NZBFILENAME)['NAME'] == \
            'A.Great.Movie.1983.DVDRip.x264-AWESOME'
        assert script.parse_nzbfile(NZBFILENAME)['MOVIEYEAR'] == '1983'
        evil_filename = join(obsfucated_dir, 'ajklada3adaadfafdkl.mkv')
        good_filename = script.deobfuscate(
            filename=evil_filename,
        )
        assert basename(good_filename) == \
            'A.Great.Movie.1983.DVDRip.x264-AWESOME.mkv'

        # Same test, but if we didn't have the NZB-Name
        script.nzb_unset('name')
        good_filename = script.deobfuscate(
            filename=evil_filename,
        )
        assert basename(good_filename) == \
            'A.Great.Movie(1983).mkv'

        # If we don't have the ProperName, it gets even more scarse
        script.nzb_unset('propername')
        good_filename = script.deobfuscate(
            filename=evil_filename,
        )
        assert splitext(basename(good_filename))[0] == \
            splitext(basename(NZBFILENAME))[0]

        del script

        # cleanup
        try:
            rmtree(download_dir)
        except:
            pass
        try:
            rmtree(obsfucated_dir)
        except:
            pass

    def test_guesses(self):
        # a NZB Logger set to False uses stderr
        script = PostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)

        guess_dict = {
            'type': 'movie',
            'title': 'A Great Title',
            'year': 1998,
            'screenSize': '720p',
            'format': 'HD-DVD',
            'audioCodec': 'DTS',
            'videoCodec': 'h264',
            'releaseGroup': 'GREATTEAM',
            'BadEntry': 'Ignored',
        }

        # Keep a handle on the real standard output
        stdout = sys.stdout
        sys.stdout = StringIO.StringIO()
        script.push_guess(guess_dict)

        # extract data
        sys.stdout.seek(0)
        output = sys.stdout.read().strip()

        # return stdout back to how it was
        sys.stdout = stdout

        guess_keys = guess_dict.keys()
        guess_keys.remove('BadEntry')
        cmp_output = ['%s%s%s%s=%s' % (
            NZBGET_MSG_PREFIX,
            PUSH_ENVIRO_ID,
            SHR_ENVIRO_GUESS_ID,
            k.upper(),
            str(guess_dict[k]),
        ) for k in guess_keys]

        output = re.split('[\r\n]+', output)
        for _str in output:
            assert _str in cmp_output

        # Retrieve content
        results = script.pull_guess()

        # Minus 1 for BadEntry that should not
        # be part of fetch
        assert len(results) == len(guess_dict) - 1
        for k, v in results.items():
            assert k in guess_dict
            assert str(guess_dict[k]) == v



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

import re

# Obfuscated Expression
OBFUSCATED_PATH_RE = re.compile(
    '^[a-z0-9]+$',
    re.IGNORECASE,
)
OBFUSCATED_FILE_RE = re.compile(
    '^[a-z0-9]+\.[a-z0-9]+$',
    re.IGNORECASE,
)


class TOTAL_STATUS(object):
    """Cumulative (Total) Status of NZB Processing
    """
    # everything OK
    SUCCESS = 'SUCCESS'
    # download is damaged but probably can be repaired; user intervention is
    # required;
    WARNING = 'WARNING'
    # download has failed or a serious error occurred during
    # post-processing (unpack, par);
    FAILURE = 'FAILURE'
    # download was deleted; post-processing scripts are usually not called in
    # this case; however it's possible to force calling scripts with command
    # "post-process again".
    DELETED = 'DELETED'

# TOTALSTATUS Delimiter
TOTALSTATUS_DELIMITER = '/'


class SCRIPT_STATUS(object):
    """Summary status of the scripts executed before the current one
    """
    # no other scripts were executed yet or all of them have ended with an exit
    # code of: NONE
    NONE = 'NONE'
    # all other scripts have ended with exit code "SUCCESS"
    SUCCESS = 'SUCCESS'
    # at least one of the script has failed
    FAILURE = 'FAILURE'


class PAR_STATUS(object):
    """This is a depricated flag (as of NZBGet v13) but previously
    provides the status of the par-check of the downloaded content.
    """
    # not checked: par-check is disabled or nzb-file does not contain
    # any par-files
    SKIPPED = 0
    # checked and failed to repair
    FAILURE = 1
    # checked and successfully repaired
    SUCCESS = 2
    # checked and can be repaired but repair is disabled
    DISABLED = 3


class UNPACK_STATUS(object):
    """This is a depricated flag (as of NZBGet v13) but previously
    provides the status of the unpacking of the downloaded content.
    """
    # unpack is disabled or was skipped due to nzb-file properties
    # or due to errors during par-check
    SKIPPED = 0
    # unpack failed
    FAILURE = 1
    # unpack was successful
    SUCCESS = 2



# -*- encoding: utf-8 -*-
#
# A Test Suite (for nose) for the MultiScripts
#
# Copyright (C) 2014-2017 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
import os

from TestBase import TestBase
from TestBase import TEMP_DIRECTORY

from nzbget.PostProcessScript import PostProcessScript
from nzbget.PostProcessScript import POSTPROC_ENVIRO_ID
from nzbget.ScanScript import ScanScript
from nzbget.ScanScript import SCAN_ENVIRO_ID
from nzbget.SchedulerScript import SchedulerScript
from nzbget.SchedulerScript import SCHEDULER_ENVIRO_ID
from nzbget.ScriptBase import SCRIPT_MODE
from nzbget.ScriptBase import EXIT_CODE
from nzbget.ScriptBase import SYS_ENVIRO_ID
from nzbget.Logger import VERY_VERBOSE_DEBUG


class TestPostProcessScript(TestBase):
    def setUp(self):
        """This method is run once before _each_ test method is executed"""
        super(TestPostProcessScript, self).setUp()

        if '%sDESTDIR' % SYS_ENVIRO_ID in os.environ:
            del os.environ['%sDESTDIR' % SYS_ENVIRO_ID]
        if '%sDIRECTORY' % SCAN_ENVIRO_ID in os.environ:
            del os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID]
        if '%sDIRECTORY' % POSTPROC_ENVIRO_ID in os.environ:
            del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

    def tearDown(self):
        """This method is run once after _each_ test method is executed"""
        # Eliminate any variables defined
        if '%sDESTDIR' % SYS_ENVIRO_ID in os.environ:
            del os.environ['%sDESTDIR' % SYS_ENVIRO_ID]
        if '%sDIRECTORY' % SCAN_ENVIRO_ID in os.environ:
            del os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID]
        if '%sDIRECTORY' % POSTPROC_ENVIRO_ID in os.environ:
            del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

    def test_dual_script01(self):
        """
        Testing NZBGet Script initialization using environment variables
        """
        class TestDualScript(PostProcessScript, SchedulerScript):
            pass

        print os.environ.keys()
        # No environment variables make us unsure what we're testing
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.NONE

        # Scheduler Sanity
        os.environ['%sTASKID' % SCHEDULER_ENVIRO_ID] = '1'
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.SCHEDULER
        del os.environ['%sTASKID' % SCHEDULER_ENVIRO_ID]

        # Scan Sanity
        os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.NONE
        del os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID]

        # PostProcess Sanity
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

        os.environ['%sDESTDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING
        del os.environ['%sDESTDIR' % SYS_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

        # Post Process still trumps all (if all are set)
        os.environ['%sDESTDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING
        del os.environ['%sDESTDIR' % SYS_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

    def test_dual_script02(self):
        """
        Testing NZBGet Script initialization using environment variables
        Reversing them from test01 alters the initialization, but
        should not affect the end result
        """
        class TestDualScript(SchedulerScript, PostProcessScript):
            pass

        # No environment variables make us unsure what we're testing
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.NONE

        # Scheduler Sanity
        os.environ['%sTASKID' % SCHEDULER_ENVIRO_ID] = '1'
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.SCHEDULER
        del os.environ['%sTASKID' % SCHEDULER_ENVIRO_ID]

        # Scan Sanity
        os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.NONE
        del os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID]

        # PostProcess Sanity
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

        os.environ['%sDESTDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING
        del os.environ['%sDESTDIR' % SYS_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

        # Post Process still trumps all (if all are set)
        os.environ['%sDESTDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING
        del os.environ['%sDESTDIR' % SYS_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

    def test_dual_script03(self):
        """
        Testing NZBGet Script initialization using environment variables
        Reversing them from test01 alters the initialization, but
        should not affect the end result
        """
        class TestDualScript(ScanScript, PostProcessScript):
            pass

        # No environment variables make us unsure what we're testing
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.NONE

        # Scheduler Sanity
        os.environ['%sDESTDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.NONE
        del os.environ['%sDESTDIR' % SYS_ENVIRO_ID]

        # Scan Sanity
        os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.SCAN
        del os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID]

        # PostProcess Sanity
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

        os.environ['%sDESTDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING
        del os.environ['%sDESTDIR' % SYS_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

        # Post Process still trumps all (if all are set)
        os.environ['%sDESTDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING
        del os.environ['%sDESTDIR' % SYS_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

    def test_dual_force_script01(self):

        class TestDualScript(ScanScript, PostProcessScript):
            pass

        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG,
                                script_mode=SCRIPT_MODE.POSTPROCESSING)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING

        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG,
                                script_mode=SCRIPT_MODE.SCAN)
        assert script.detect_mode() == SCRIPT_MODE.SCAN

        # SchedulerScript is not part of DualScript, so therefore
        # it can't be foreced
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG,
                                script_mode=SCRIPT_MODE.SCHEDULER)
        assert script.detect_mode() == SCRIPT_MODE.NONE

        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG,
                                script_mode=SCRIPT_MODE.NONE)
        assert script.detect_mode() == SCRIPT_MODE.NONE

    def test_dual_force_script02(self):

        class TestDualScript(SchedulerScript, PostProcessScript):
            pass

        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG,
                                script_mode=SCRIPT_MODE.POSTPROCESSING)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING

        # ScanScript is not part of DualScript, so therefore
        # it can't be foreced
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG,
                                script_mode=SCRIPT_MODE.SCAN)
        assert script.detect_mode() == SCRIPT_MODE.NONE

        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG,
                                script_mode=SCRIPT_MODE.SCHEDULER)
        assert script.detect_mode() == SCRIPT_MODE.SCHEDULER

        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG,
                                script_mode=SCRIPT_MODE.NONE)
        assert script.detect_mode() == SCRIPT_MODE.NONE

    def test_dual_force_script03(self):

        class TestDualScript(SchedulerScript, ScanScript):
            pass

        # PostProcessingScript is not part of DualScript, so therefore
        # it can't be foreced
        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG,
                                script_mode=SCRIPT_MODE.POSTPROCESSING)
        assert script.detect_mode() == SCRIPT_MODE.NONE

        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG,
                                script_mode=SCRIPT_MODE.SCAN)
        assert script.detect_mode() == SCRIPT_MODE.SCAN

        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG,
                                script_mode=SCRIPT_MODE.SCHEDULER)
        assert script.detect_mode() == SCRIPT_MODE.SCHEDULER

        script = TestDualScript(logger=False, debug=VERY_VERBOSE_DEBUG,
                                script_mode=SCRIPT_MODE.NONE)
        assert script.detect_mode() == SCRIPT_MODE.NONE

    def test_tri_script01(self):
        """
        Testing NZBGet Script initialization using environment variables
        Reversing them from test01 alters the initialization, but
        should not affect the end result
        """
        class TestTriScript(SchedulerScript, PostProcessScript, ScanScript):
            pass

        # No environment variables make us unsure what we're testing
        script = TestTriScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.NONE

        # Scheduler Sanity
        os.environ['%sTASKID' % SCHEDULER_ENVIRO_ID] = '1'
        script = TestTriScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.SCHEDULER
        del os.environ['%sTASKID' % SCHEDULER_ENVIRO_ID]

        # Scan Sanity
        os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestTriScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.SCAN
        del os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID]

        os.environ['%sDESTDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestTriScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.SCAN
        del os.environ['%sDESTDIR' % SYS_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID]

        # PostProcess Sanity
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestTriScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

        os.environ['%sDESTDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestTriScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING
        del os.environ['%sDESTDIR' % SYS_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

        # Post Process still trumps all (if all are set)
        os.environ['%sDESTDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = TEMP_DIRECTORY
        script = TestTriScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.detect_mode() == SCRIPT_MODE.POSTPROCESSING
        del os.environ['%sDESTDIR' % SYS_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % SCAN_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID]

    def test_dual_run(self):
        class TestDualScript(SchedulerScript, PostProcessScript):
            def postprocess_main(self, *args, **kwargs):
                return None

            def scheduler_main(self, *args, **kwargs):
                return False
        script = TestDualScript(
            logger=False, debug=VERY_VERBOSE_DEBUG,
            script_mode=SCRIPT_MODE.POSTPROCESSING,
        )
        assert script.run() == EXIT_CODE.NONE

        script = TestDualScript(
            logger=False, debug=VERY_VERBOSE_DEBUG,
            script_mode=SCRIPT_MODE.SCHEDULER,
        )
        assert script.run() == EXIT_CODE.FAILURE
